package de.fesenmeyer.dbnormalizer.core;

/**
 * DBConfig holds information needed to access a database with a JDBC driver.
 * 
 * @author DF
 * 
 */
public class DBConfig {
	/**
	 * The (JDBC) database url.
	 */
	private String dbURL = null;
	/**
	 * The username.
	 */
	private String username = null;
	/**
	 * The password.
	 */
	private String password = null;
	/**
	 * The name of the JDBC-driver class.
	 */
	private String driverClassName = null;
	/**
	 * The (absolute or relative) file name of the JDBC-driver jar file.
	 */
	private String driverFileName = null;

	/**
	 * Class constructor specifying all properties.
	 * @param dbURL the (JDBC) database url 
	 * @param username the username (can be null)
	 * @param password the password (can be null)
	 * @param driverClassName the name of the JDBC-driver class
	 * @param driverFileName the (absolute or relative) file name of the JDBC-driver jar file
	 */
	public DBConfig(String dbURL, String username, String password, String driverClassName,
			String driverFileName) {
		super();
		this.dbURL = dbURL;
		this.username = username;
		this.password = password;
		this.driverClassName = driverClassName;
		this.driverFileName = driverFileName;
	}

	/**
	 * Gets the (JDBC) database url.
	 * @return the (JDBC) database url
	 */
	public String getDbURL() {
		return dbURL;
	}

	/**
	 * Sets the (JDBC) database url.
	 * @param dbURL the (JDBC) database url
	 */
	public void setDbURL(String dbURL) {
		this.dbURL = dbURL;
	}

	/**
	 * Gets the name of the JDBC-driver class
	 * @return name of the JDBC-driver class
	 */
	public String getDriverClassName() {
		return driverClassName;
	}

	/**
	 * Sets the name of the JDBC-driver class.
	 * @param driverClassName the name of the JDBC-driver class
	 */
	public void setDriverClassName(String driverClassName) {
		this.driverClassName = driverClassName;
	}

	/**
	 * Gets the (absolute or relative) file name of the JDBC-driver jar file.
	 * @return the (absolute or relative) file name of the JDBC-driver jar file
	 */
	public String getDriverFileName() {
		return driverFileName;
	}

	/**
	 * Sets the (absolute or relative) file name of the JDBC-driver jar file.
	 * @param driverFileName the (absolute or relative) file name of the JDBC-driver jar file
	 */
	public void setDriverFileName(String driverFileName) {
		this.driverFileName = driverFileName;
	}

	/**
	 * Gets the password.
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * Sets the password.
	 * @param password the password
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * Gets the username.
	 * @return the username
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * Sets the username
	 * @param username the username
	 */
	public void setUsername(String username) {
		this.username = username;
	}

}
