package de.fesenmeyer.dbnormalizer.core;

import java.util.Comparator;

/**
 * <p>
 * Comparator implementation for sorting FDs
 * alphabetically. The implementation compares first
 * the LHS of the FDs, then the RHS.
 * </p>
 * <p>
 * Consider the following unordered set of FDs:
 * <pre>
 * B -> F
 * A,B -> E
 * A,B -> D
 * </pre>
 * The use of this comparator for sorting these FDs
 * has the following result:
 * <pre>
 * A,B -> D
 * A,B -> E
 * B -> F
 * </pre>
 * </p>
 * @author DF
 *
 */
public class FDAlphabeticalComparator implements Comparator<FD> {

	/* (non-Javadoc)
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	public int compare(FD o1, FD o2) {
		int compareVal = 0;
		if (o1 == null || o2 == null) {
			throw new IllegalArgumentException("None of the FDs to be compared must be null!");
		}
		AttributeStringSet lhs1 = o1.getLHS();
		AttributeStringSet lhs2 = o2.getLHS();
		compareVal = lhs1.compareTo(lhs2);

		if (compareVal == 0) {
			AttributeStringSet rhs1 = o1.getRHS();
			AttributeStringSet rhs2 = o2.getRHS();
			compareVal = rhs1.compareTo(rhs2);
		}

		return compareVal;
	}

}
