package de.fesenmeyer.dbnormalizer.core;

import java.util.ArrayList;
import java.util.List;

/**
 * Abstract base class for keys of a table. The attributes the
 * key consists of are represented as a list, not as a set,
 * because the order of the attributes is not arbitrary for
 * a key. For example, when defining a foreign key (FK), the order
 * of the referenced table's PK attributes must be considered.
 * @author DF
 *
 */
public abstract class Key extends AbstractDBItem {

	/**
	 * The attributes this key consists of.
	 */
	private List<String> attributes = new ArrayList<String>();

	/**
	 * Class Constructor.
	 */
	public Key() {
		super();
	}

	/**
	 * Gets the attributes this key consists of.
	 * @return the attributes
	 */
	public List<String> getAttributes() {
		return attributes;
	}

	/**
	 * Sets the attributes this key consists of.
	 * @param attributes the attributes
	 */
	public void setAttributes(List<String> attributes) {
		this.attributes = attributes;
	}

	/**
	 * Overridden method which returns an appropriate
	 * string representation of this key.
	 * @return the string representation of this key
	 */
	@Override
	public String toString() {
		StringBuffer strBuf = new StringBuffer();
		if (getName() != null) {
			strBuf.append(getName()).append(" ");
		}
		strBuf.append(this.attributes.toString());
		return strBuf.toString();
	}

}