package de.fesenmeyer.dbnormalizer.core.algorithms;

import de.fesenmeyer.dbnormalizer.core.DBNormalizerException;
import de.fesenmeyer.dbnormalizer.core.Table;

/**
 * Abstract base class for normalization algorithms.
 * @author DF
 *
 */
public abstract class AbstractNormalizationAlgorithm {

	/**
	 * <p>
	 * Creates a normalization proposal for the Table specified by the argument oldTable.
	 * </p>
	 * <p>
	 * <b>NOTE</b>: Before applying this method to a table, make sure that it contains
	 * all the necessary data such as minimal cover ({@link Table#setFDMinCover(java.util.Set)}),
	 * candidate keys ({@link Table#setCandKeys(java.util.Set)})  and primary key 
	 * ({@link Table#setPk(de.fesenmeyer.dbnormalizer.core.PK)}). The methods for calculating
	 * this data can be found in the {@link FDAlgorithms} class.
	 * </p>
	 * @param oldTable the Table, for which a normalization proposal should be created
	 * @return a normalization proposal
	 * @throws DBNormalizerException if the table object does not supply all of the data
	 * needed for normalization or if another error occurs.
	 */
	public abstract NormalizationProposal normalize(Table oldTable) throws DBNormalizerException;

}