package de.fesenmeyer.dbnormalizer.core.algorithms;

import java.util.Map;
import java.util.Set;

import de.fesenmeyer.dbnormalizer.core.AttributeStringSet;
import de.fesenmeyer.dbnormalizer.core.FD;
import de.fesenmeyer.dbnormalizer.core.NormalForm;
import de.fesenmeyer.dbnormalizer.core.Table;

/**
 * Class holding the result of the normal form determination performed
 * by the class {@link NFDetermination}.
 * @author DF
 * @see NFDetermination
 *
 */
public class NFDeterminationResult {

	/**
	 * The candidate keys used for normal form determination.
	 */
	private Set<AttributeStringSet> candidateKeys = null;
	/**
	 * The table.
	 */
	private Table table = null;
	
	/**
	 * The FDs used for normal form determination.
	 */
	private Set<FD> fds = null;

	/**
	 * The determined normal form.
	 */
	private NormalForm nf = null;
	/**
	 * The determined violating FDs, grouped by normal form.
	 */
	private Map<NormalForm, Set<FD>> violatingFDs = null;

	/**
	 * Class constructor.
	 * @param candidateKeys the candidate keys used for normal form determination
	 * @param table the table
	 * @param fds the FDs used
	 * @param nf the determined normal form
	 * @param violatingFDs the determined violating FDs, grouped by normal form
	 */
	public NFDeterminationResult(Set<AttributeStringSet> candidateKeys, Table table, Set<FD> fds,
			NormalForm nf, Map<NormalForm, Set<FD>> violatingFDs) {
		this.candidateKeys = candidateKeys;
		this.table = table;
		this.fds = fds;
		this.nf = nf;
		this.violatingFDs = violatingFDs;
	}

	/**
	 * Gets the candidate keys used for normal form determination.
	 * @return the candidate keys
	 */
	public Set<AttributeStringSet> getCandidateKeys() {
		return candidateKeys;
	}

	/**
	 * Gets the FDs used for normal form determination.
	 * @return the FDs
	 */
	public Set<FD> getFds() {
		return fds;
	}

	/**
	 * Gets the determined normal form.
	 * @return the normal form
	 */
	public NormalForm getNf() {
		return nf;
	}

	/**
	 * Gets the table.
	 * @return the table
	 */
	public Table getTable() {
		return table;
	}

	/**
	 * Gets the determined violating FDs, grouped by normal form.
	 * @return the violating FDs.
	 */
	public Map<NormalForm, Set<FD>> getViolatingFDs() {
		return violatingFDs;
	}
}
