package de.fesenmeyer.dbnormalizer.core.algorithms;

import java.util.ArrayList;
import java.util.Collection;

import de.fesenmeyer.dbnormalizer.core.FK;

/**
 * Proposal of foreign keys (FKs) to be defined between the created
 * tables of a normalization proposal.
 * @author DF
 * @see NormalizationFKsDetermination
 */
public class NormalizationFKsProposal {
	/**
	 * The FKs to be defined 
	 * between the created tables of the normalization proposal.
	 */
	private Collection<FK> innerFKS = new ArrayList<FK>();
	/**
	 * The FKs to be defined 
	 * due to the old table's imported keys.
	 */
	private Collection<FK> importedFKs = new ArrayList<FK>();
	/**
	 * The FKs to be defined 
	 * due to the old table's exported keys.
	 */
	private Collection<FK> exportedFKs = new ArrayList<FK>();
	/**
	 * The FKs to be defined 
	 * due to FKs between the old table's attributes ("hierarchical FKs").
	 */
	private Collection<FK> hierarchicalFKs = new ArrayList<FK>();

	/**
	 * Class Constructor.
	 */
	public NormalizationFKsProposal() {
	}

	/**
	 * Gets the FKs to be defined 
	 * between the created tables of the normalization proposal.
	 * @return a collection of FKs
	 */
	public Collection<FK> getInnerFKS() {
		return innerFKS;
	}

	/**
	 * Sets the FKs to be defined 
	 * between the created tables of the normalization proposal.
	 * @param innerFKS a collection of FKs
	 */
	public void setInnerFKS(Collection<FK> innerFKS) {
		this.innerFKS = innerFKS;
	}

	/**
	 * Gets the FKs to be defined 
	 * due to the old table's imported keys.
	 * @return a collection of FKs
	 */
	public Collection<FK> getImportedFKs() {
		return importedFKs;
	}

	/**
	 * Sets the FKs to be defined 
	 * due to the old table's imported keys.
	 * @param importedFKs a collection of FKs
	 */
	public void setImportedFKs(Collection<FK> importedFKs) {
		this.importedFKs = importedFKs;
	}

	/**
	 * Gets the FKs to be defined 
	 * due to the old table's exported keys.
	 * @return a collection of FKs
	 */
	public Collection<FK> getExportedFKs() {
		return exportedFKs;
	}

	/**
	 * Sets the FKs to be defined 
	 * due to the old table's exported keys.
	 * @param exportedFKs a collection of FKs
	 */
	public void setExportedFKs(Collection<FK> exportedFKs) {
		this.exportedFKs = exportedFKs;
	}

	/**
	 * Gets the FKs to be defined 
	 * due to FKs between the old table's attributes ("hierarchical FKs").
	 * @return a collection of FKs
	 */
	public Collection<FK> getHierarchicalFKs() {
		return hierarchicalFKs;
	}

	/**
	 * Sets the FKs to be defined 
	 * due to FKs between the old table's attributes ("hierarchical FKs").
	 * @param hierarchicalFKs a collection of FKs
	 */
	public void setHierarchicalFKs(Collection<FK> hierarchicalFKs) {
		this.hierarchicalFKs = hierarchicalFKs;
	}

}
