package de.fesenmeyer.dbnormalizer.core.configuration;

import java.util.ArrayList;
import java.util.Collection;

import de.fesenmeyer.dbnormalizer.core.DBConfig;

/**
 * The main configuration class which holds all the data for reading
 * or writing a session.
 * @author DF
 *
 */
public class SessionConfig {
	/**
	 * The DB-Configuration, may be null if the session
	 * does not access a DB.
	 */
	private DBConfig dbConfig = null;

	/**
	 * The configuration filename.
	 */
	private String configFileName = null;

	/**
	 * A collection of TableContainerConfig objects.
	 */
	private Collection<TableContainerConfig> tableContainerConfigs = new ArrayList<TableContainerConfig>();

	/**
	 * Class constructor specifying the configuration file name.
	 * @param configFileName the configuration file name
	 */
	public SessionConfig(String configFileName) {
		this.configFileName = configFileName;
	}

	/**
	 * Gets the configuration file name
	 * @return the configuration file name
	 */
	public String getConfigFileName() {
		return configFileName;
	}

	/**
	 * Sets the configuration file name.
	 * @param configFileName the configuration file name
	 */
	public void setConfigFileName(String configFileName) {
		this.configFileName = configFileName;
	}

	/**
	 * Gets the DB-Configuration.
	 * @return the DB-Configuration, if the session accesses a DB;
	 * <code>null</code>, otherwise
	 */
	public DBConfig getDbConfig() {
		return dbConfig;
	}

	/**
	 * Sets the DB-Configuration
	 * @param dbConfig the DB-Configuration
	 */
	public void setDbConfig(DBConfig dbConfig) {
		this.dbConfig = dbConfig;
	}

	/**
	 * Gets the TableContainer configuration objects.
	 * @return the TableContainer configuration objects
	 * @see TableContainerConfig
	 */
	public Collection<TableContainerConfig> getTableContainerConfigs() {
		return tableContainerConfigs;
	}

	
	/**
	 * Sets the TableContainer configuration objects
	 * @param tableContainerConfigs the TableContainer configuration objects
	 * @see TableContainerConfig
	 */
	public void setTableContainerConfigs(Collection<TableContainerConfig> tableContainerConfigs) {
		this.tableContainerConfigs = tableContainerConfigs;
	}

}
