package de.fesenmeyer.dbnormalizer.core.configuration.xml;

import java.io.File;

/**
 * Utility class for retrieving file names for XML-Schema- and XLST-Files
 * needed by DBNormalizer. This is the single place to configure the location
 * of the files needed to read or write configuration files.
 * @author DF
 *
 */
public final class DBNormalizerXMLPathUtil {

	/**
	 * The path of the DBNormalizer-Schema-File relative to the working directory.
	 */
	private static final String DBNORMALIZER_SCHEMA_FILENAME = "xsd/configuration.xsd";
	
	/**
	 * The path of the TaneJava-to-DBNormalizer-Transformation-File relative to the working directory.
	 */
	private static final String TANEJAVA2DBN_XSL_FILENAME = "xsl/transform_tanejava2dbn.xsl";
	
	/**
	 * Makes the constructor unaccessible.
	 */
	private DBNormalizerXMLPathUtil() {
	}
	
	/**
	 * Gets the (absolute) path of the DBNormalizer-Schema-File.
	 * @return the (absolute) path of the DBNormalizer-Schema-File
	 */
	public static String getDBNormalizerSchemaFilename() {
		return new File(DBNORMALIZER_SCHEMA_FILENAME).getAbsolutePath();
	}
	
	/**
	 * Gets the (absolute) path of the TaneJava-to-DBNormalizer-Transformation-File.
	 * @return the (absolute) path of the TaneJava-to-DBNormalizer-Transformation-File
	 */
	public static String getTaneJava2DBNormalizerXSLFilename() {
		return new File(TANEJAVA2DBN_XSL_FILENAME).getAbsolutePath();
	}
}
