package de.fesenmeyer.dbnormalizer.core.configuration.xml;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

/**
 * Class which provides writing an arbitrary DOM-document
 * to an OutputStream, e.g. a file.
 * @author DF
 * @see Document
 */
public class DOMDocumentWriter {

	/**
	 * The DOM-document.
	 */
	private Document doc = null;

	/**
	 * Class constructor specifying the DOM-document to be written
	 * to a file.
	 * @param doc the DOM-document
	 */
	public DOMDocumentWriter(Document doc) {
		this.doc = doc;
	}

	/**
	 * Writes the DOM-Document to the OutputStream specified by
	 * argument oStream.
	 * @param oStream the OutputStream to which should be written
	 * @throws TransformerException if a XML-Transformer specific exception occurs
	 * (e.g. if no TransformerFactory is available).
	 */
	public void write(OutputStream oStream) throws TransformerException {

		TransformerFactory tFactory = TransformerFactory.newInstance();
		Transformer transformer;

		transformer = tFactory.newTransformer();
		transformer.setErrorListener(new SimpleErrorListener());
		transformer.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");

		//indent ("pretty-print")
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		// Unless a width is set, there will be only line breaks but no indentation.
		// The IBM JDK and the Sun JDK don't agree on the property name,
		// so we set them both.
		transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "4");
		transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
		
		DOMSource source = new DOMSource(doc);
		StreamResult result = new StreamResult(oStream);
		transformer.transform(source, result);
	}

	/**
	 * Writes the DOM-Document to the file specified by the
	 * argument fileName.
	 * @param fileName the path of the file to which should be written
	 * @throws TransformerException if a XML-Transformer specific exception occurs
	 * (e.g. if no TransformerFactory is available).
	 * @throws IOException if an IO-specific exception occurs (e.g. if there are
	 * no rights to write the file).
	 */
	public void write(String fileName) throws TransformerException, IOException {
		FileOutputStream foStream = null;
		foStream = new FileOutputStream(fileName);

		write(foStream);
		foStream.flush();
		foStream.close();
	}
}
