package de.fesenmeyer.dbnormalizer.core.configuration.xml;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;

/**
 * Class which performs a XSL-Transformation on
 * a DOM-Document.
 * @author DF
 *
 */
public class DOMTransformer {

	/**
	 * The XSLT-File.
	 */
	private String xsltFile = null;

	/**
	 * The Input-DOM-Document.
	 */
	private Document inputDoc = null;

	/**
	 * Class constructor specifying the DOM-Document which should
	 * be used as input and the XSLT-file to be used for the transformation.
	 * @param doc the Input-DOM-Document
	 * @param xsltFile the XSLT-file
	 */
	public DOMTransformer(Document doc, String xsltFile) {
		this.inputDoc = doc;
		this.xsltFile = xsltFile;
	}

	/**
	 * Performs the XSL-Transformation and returns the
	 * Output-DOM-Document.
	 * @return the Output-DOM-Document
	 * @throws TransformerException if a XML-Transformer specific exception occurs
	 * (e.g. if no TransformerFactory is available). 
	 */
	public Document transform() throws TransformerException {

		TransformerFactory tFactory = TransformerFactory.newInstance();
		Transformer transformer;

		// transform with given xslt-file
		Source xsltSource = new StreamSource(xsltFile);
		transformer = tFactory.newTransformer(xsltSource);
		
		transformer.setErrorListener(new SimpleErrorListener());
		
		DOMSource source = new DOMSource(inputDoc);
		DOMResult result = new DOMResult();

		transformer.transform(source, result);

		return (Document) result.getNode();

	}
}
