package de.fesenmeyer.dbnormalizer.core.configuration.xml;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * Class providing utility methods for retrieving
 * information from DOM-Documents.
 * @author DF
 *
 */
public class DOMUtil {
	/**
	 * Gets the first descendant of the element
	 * specified by argument element which has the tag name
	 * specified by argument name.
	 * @param element an element
	 * @param name a tag name
	 * @return the element, if one could be found;
	 * <code>null</code>, otherwise
	 */
	public static Element getFirstElement(Element element, String name) {
		NodeList nl = element.getElementsByTagName(name);
		if (nl.getLength() < 1) {
			return null;
		}
		return (Element) nl.item(0);
	}

	/**
	 * Gets the first child of the element
	 * specified by argument element which has the tag name
	 * specified by argument name. Descendants other than children
	 * are ignored - in contrast to the {@link #getFirstElement(Element, String)}
	 * method.
	 * @param element an element
	 * @param name a tag name
	 * @return the element, if one could be found;
	 * <code>null</code>, otherwise
	 */
	public static Element getFirstChildElement(Element element, String name) {
		NodeList nl = element.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			Node node = nl.item(i);
			if (node instanceof Element) {
				Element nodeElement = (Element) node;
				if (name.equals(nodeElement.getTagName())) {
					return nodeElement;
				}
			}
		}
		return null;
	}

	/**
	 * Gets the text content of the first descendant of the element
	 * specified by argument element which has the tag name
	 * specified by argument name.
	 * @param element an element
	 * @param name a tag name
	 * @return the text content as a string, if such an descendant could be found;
	 * <code>null</code>, otherwise
	 */
	public static String getSimpleElementText(Element element, String name) {
		Element namedElement = getFirstElement(element, name);
		if (namedElement == null) {
			return null;
		} else {
			return getSimpleElementText(namedElement);
		}
	}

	/**
	 * Gets all child text nodes of an element as
	 * a single string.
	 * @param element the element
	 * @return a string consisting of all the child text nodes
	 */
	public static String getSimpleElementText(Element element) {
		StringBuffer sb = new StringBuffer();
		NodeList children = element.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (child instanceof Text)
				sb.append(child.getNodeValue());
		}
		return sb.toString();
	}
}
