package de.fesenmeyer.dbnormalizer.core.configuration.xml;

import java.io.File;
import java.io.IOException;

import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;



/**
 * Class which provides the possibility to validate
 * a DOM-Document against a XML-Schema.
 * @author DF
 *
 */
public class DOMValidator {

	/**
	 * The schema's filename.
	 */
	private String schemaFile = null;

	/**
	 * The DOM-Document.
	 */
	private Document doc = null;

	/**
	 * Class constructor specifying DOM-Document
	 * and the filename of the XML-Schema to be used.
	 * @param doc the DOM-Document
	 * @param schemaFile the XML-Schema's filename
	 */
	public DOMValidator(Document doc, String schemaFile) {
		this.doc = doc;
		this.schemaFile = schemaFile;
	}

	/**
	 * Validates the DOM-Document against the XML-Schema,
	 * as specified in the constructor.
	 * @throws SAXException if the XML-validation fails, e.g. if
	 * the DOM-Document is not well-formed or not valid.
	 * @throws IOException (very unlikely to occur)
	 */
	public void validate() throws SAXException, IOException {
		SchemaFactory factory = SchemaFactory
				.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

		Source schemaSource = new StreamSource(new File(schemaFile));
		Schema schema;

		schema = factory.newSchema(schemaSource);


		// create a Validator instance, which can be used to validate a document
		Validator validator = schema.newValidator();
		
		validator.setErrorHandler(new SimpleErrorHandler());

		validator.validate(new DOMSource(doc));
		
	}

}
