package de.fesenmeyer.dbnormalizer.core.dbaccess;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * Class which provides saving a SQL-ResultSet in a matrix consisting of
 * rows and columns. The resultset's column names are also saved.
 * @author DF
 * 
 */
public class DataSet {

	/**
	 * The column names.
	 */
	List<String> columnNames = new ArrayList<String>();

	/**
	 * A matrix of objects.
	 */
	List<List<Object>> dataList = new ArrayList<List<Object>>();

	/**
	 * The number of columns.
	 */
	int columnCount = 0;

	/**
	 * The number of rows.
	 */
	int rowCount = 0;

	/**
	 * Class constructor specifying a ResultSet by argument rs. Fills
	 * this instance with the ResultSet's data.
	 * @param rs the ResultSet
	 * @throws DBException if a SQL-Exception occurs while reading the data
	 */
	public DataSet(ResultSet rs) throws DBException {
		try {
			ResultSetMetaData rsMd = rs.getMetaData();
			columnCount = rsMd.getColumnCount();

			// fill columnNames map
			for (int i = 0; i < columnCount; i++) {
				String columnName = rsMd.getColumnName(i + 1);
				columnNames.add(columnName);
			}

			// fill the data list
			while (rs.next()) {
				ArrayList<Object> rowDataList = new ArrayList<Object>();
				for (int i = 0; i < columnCount; i++) {
					Object data = rs.getObject(i + 1);
					rowDataList.add(data);
				}
				dataList.add(rowDataList);
				rowCount++;
			}
		} catch (SQLException e) {
			throw new DBException(e);
		}
	}

	/**
	 * Get the row at the specified index.
	 * @param index the Index
	 * @return the row as a list of objects
	 */
	public List<Object> getRow(int index) {
		return dataList.get(index);
	}

	/**
	 * Gets the column names.
	 * @return the column names as a list of strings
	 */
	public List<String> getColumnNames() {
		return columnNames;
	}

	/**
	 * Gets the number of columns.
	 * @return the number of columns
	 */
	public int getColumnCount() {
		return columnCount;
	}

	/**
	 * Gets the number of rows.
	 * @return the number of rows
	 */
	public int getRowCount() {
		return rowCount;
	}
}
