package de.fesenmeyer.dbnormalizer.core.util;

import java.util.Set;

/**
 * Provides some commonly needed set operations.
 * 
 * @author DF
 *
 */
public final class SetOperationsUtil {

	/**
	 * Tests if argument set1 is a subset of argument set2.
	 * 
	 * @param <T> an arbitrary type
	 * @param set1 a Set
	 * @param set2 a Set
	 * @return <code>true</code>, if set1 is a subset of set2; 
	 * <code>false</code>, otherwise
	 */
	public static <T> boolean isSubset(Set<T> set1, Set<T> set2) {
		return set2.containsAll(set1);
	}

	/**
	 * Tests if argument set1 is a proper subset of argument set2.
	 * 
	 * @param <T> an arbitrary type
	 * @param set1 a Set
	 * @param set2 a Set
	 * @return <code>true</code>, if set1 is a proper subset of set2; 
	 * <code>false</code>, otherwise
	 */
	public static <T> boolean isProperSubset(Set<T> set1, Set<T> set2) {
		boolean isSubset = isSubset(set1, set2);
		boolean isProperSubset = isSubset && (set1.size() < set2.size());
		return isProperSubset;
	}

	/**
	 * Tests if argument set1 contains set2 as a subset.
	 * @param <T> an arbitrary type
	 * @param set1 a Set
	 * @param set2 a Set
	 * @return <code>true</code>, if set1 contains set2 as a subset;
	 * <code>false</code>, otherwise
	 */
	public static <T> boolean containsSubset(Set<Set<T>> set1, Set<T> set2) {
		boolean containsSubset = false;
		for (Set<T> entry1 : set1) {
			if (isSubset(set2, entry1)) {
				containsSubset = true;
				break;
			}
		}
		return containsSubset;
	}
}
