package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import de.fesenmeyer.dbnormalizer.core.DBConfig;
import de.fesenmeyer.dbnormalizer.core.log.DBNormalizerLogger;
import de.fesenmeyer.dbnormalizer.core.util.StringUtil;
import de.fesenmeyer.dbnormalizer.gui.util.DBNormalizerColors;
import de.fesenmeyer.dbnormalizer.gui.util.FileFilters;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class DBConfigFrame extends JDialog implements ActionListener {

	private static DBNormalizerLogger logger = DBNormalizerLogger.getLogger(DBConfigFrame.class);

	private JTextField dbUrlTf = null;

	private JTextField driverClassNameTf = null;

	private JTextField driverFileNameTf = null;

	private JTextField usernameTf = null;

	private JPasswordField passwordTf = null;

	private JButton okBtn = new JButton("OK");

	private JButton cancelBtn = new JButton("Cancel");

	private DBConfig dbConfig = null;

	private boolean accepted = false;
	
	private JFileChooser jarFileChooser = null;
	
	private JButton chooseFileButton = null;

	public DBConfigFrame(JFrame owner, boolean editable) {
		super(owner, true);
		if (editable) {
			this.setTitle("Enter DB Configuration");
		} else {
			this.setTitle("DB Configuration");
		}

		Dimension dim = new Dimension(500, 300);
		this.setSize(dim);
		this.setMinimumSize(dim);

		JPanel mainPanel = (JPanel) this.getContentPane();

		JPanel editPanel = new JPanel(new GridBagLayout());
		editPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);

		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		JLabel dbUrlLbl = new JLabel("DB URL:");
		dbUrlLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		editPanel.add(dbUrlLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		this.dbUrlTf = new JTextField(40);
		this.dbUrlTf.setEditable(editable);
		editPanel.add(this.dbUrlTf, c);

		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		c.weightx = 0;
		JLabel driverClassNameLbl = new JLabel("DriverClassName:");
		driverClassNameLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		editPanel.add(driverClassNameLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		this.driverClassNameTf = new JTextField(40);
		this.driverClassNameTf.setEditable(editable);
		editPanel.add(this.driverClassNameTf, c);

		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 1;
		c.weightx = 0;
		JLabel driverFileNameLbl = new JLabel("Driver Filename:");
		driverFileNameLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		editPanel.add(driverFileNameLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.weightx = 1;
		this.driverFileNameTf = new JTextField(40);
		this.driverFileNameTf.setEditable(editable);
		editPanel.add(this.driverFileNameTf, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 0;
		this.chooseFileButton = new JButton(
				IconManager.getInstance().getIcon(IconManager.FOLDER_ICON));
		this.chooseFileButton.addActionListener(this);
		this.chooseFileButton.setEnabled(editable);
		editPanel.add(this.chooseFileButton, c);

		c.gridx = 0;
		c.gridy = 3;
		c.gridwidth = 1;
		c.weightx = 0;
		JLabel usernameLbl = new JLabel("Username:");
		usernameLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		editPanel.add(usernameLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		this.usernameTf = new JTextField(40);
		this.usernameTf.setEditable(editable);
		editPanel.add(this.usernameTf, c);

		c.gridx = 0;
		c.gridy = 4;
		c.gridwidth = 1;
		c.weightx = 0;
		JLabel passwordLbl = new JLabel("Password:");
		passwordLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		editPanel.add(passwordLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		this.passwordTf = new JPasswordField(40);
		this.passwordTf.setEditable(editable);
		editPanel.add(this.passwordTf, c);

		mainPanel.add(editPanel, BorderLayout.CENTER);

		if (editable) {
			JPanel buttonPanel = new JPanel();

			buttonPanel.add(okBtn);
			buttonPanel.add(cancelBtn);
			mainPanel.add(buttonPanel, BorderLayout.PAGE_END);

			// register action listener for buttons
			okBtn.addActionListener(this);
			cancelBtn.addActionListener(this);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == okBtn) {
			if (validateData()) {
				this.dbConfig = new DBConfig(dbUrlTf.getText(), usernameTf.getText(), String
						.valueOf(passwordTf.getPassword()), driverClassNameTf.getText(),
						driverFileNameTf.getText());
				accepted = true;
				setVisible(false);
			}
		} else if (e.getSource() == cancelBtn) {
			setDbConfig(this.dbConfig);
			accepted = false;
			setVisible(false);
		} else if (e.getSource() == chooseFileButton) {
			int retVal = this.getJarFileChooser().showOpenDialog(this);
			if (retVal == JFileChooser.APPROVE_OPTION) {
				String fileName = this.getJarFileChooser().getSelectedFile().getAbsolutePath();
				this.driverFileNameTf.setText(fileName);
			}
		}
	}

	public DBConfig getDbConfig() {
		return dbConfig;
	}

	public void setDbConfig(DBConfig dbConfig) {
		this.dbConfig = dbConfig;
		if (dbConfig != null) {
			this.dbUrlTf.setText(dbConfig.getDbURL());
			this.driverClassNameTf.setText(dbConfig.getDriverClassName());
			this.driverFileNameTf.setText(dbConfig.getDriverFileName());
			this.usernameTf.setText(dbConfig.getUsername());
			this.passwordTf.setText(dbConfig.getPassword());
		}
	}

	public boolean isAccepted() {
		return accepted;
	}

	private boolean validateData() {
		boolean ok = true;
		if (StringUtil.isBlank(this.dbUrlTf.getText())) {
			logger.error("DB URL must not be empty");
			ok = false;
		}
		if (StringUtil.isBlank(this.driverClassNameTf.getText())) {
			logger.error("DriverClassName must not be empty");
			ok = false;
		}
		if (StringUtil.isBlank(this.driverFileNameTf.getText())) {
			logger.error("Driver Filename must not be empty");
			ok = false;
		}
		return ok;
	}

	/**
	 * @return the jarFileChooser
	 */
	private JFileChooser getJarFileChooser() {
		if (this.jarFileChooser == null) {
			this.jarFileChooser = new JFileChooser();
			this.jarFileChooser.setFileFilter(FileFilters.getJARFileFilter());
		}
		return this.jarFileChooser;
	}

}
