package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.tree.TreePath;

import de.fesenmeyer.dbnormalizer.core.AbstractDBItem;
import de.fesenmeyer.dbnormalizer.core.DBNormalizerSession;
import de.fesenmeyer.dbnormalizer.core.Table;
import de.fesenmeyer.dbnormalizer.core.dbaccess.DatabaseMetadataUtil;
import de.fesenmeyer.dbnormalizer.core.log.DBNormalizerLogger;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class DBTableSelectionFrame extends JDialog implements ActionListener {

	private static DBNormalizerLogger logger = DBNormalizerLogger
			.getLogger(DBTableSelectionFrame.class);

	private DBNormalizerSession session = null;

	private DBTree dbTree = null;

	private JButton okButton = null;

	private JButton cancelButton = null;

	private List<AbstractDBItem> selectedItems = new ArrayList<AbstractDBItem>();

	private boolean accepted = false;

	public DBTableSelectionFrame(JFrame owner) {
		super(owner, true);

		this.setTitle("Select Tables and/or whole containers");

		Dimension dim = new Dimension(440, 600);
		this.setSize(dim);
		this.setMinimumSize(dim);

		JPanel mainPanel = (JPanel) this.getContentPane();
		mainPanel.setLayout(new BorderLayout());

		this.dbTree = new DBTree();
		JScrollPane scrollPane = new JScrollPane(this.dbTree);
		mainPanel.add(scrollPane, BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();

		this.okButton = new JButton("OK");
		this.okButton.addActionListener(this);
		buttonPanel.add(this.okButton);

		this.cancelButton = new JButton("Cancel");
		this.cancelButton.addActionListener(this);
		buttonPanel.add(this.cancelButton);

		mainPanel.add(buttonPanel, BorderLayout.SOUTH);

	}

	public void actionPerformed(ActionEvent e) {
		this.selectedItems = new ArrayList<AbstractDBItem>();
		if (e.getSource() == okButton) {
			TreePath[] selPaths = this.dbTree.getSelectionPaths();
			for (TreePath path : selPaths) {
				DBNormalizerTreeNode node = (DBNormalizerTreeNode) path.getLastPathComponent();
				DBNormalizerTreeNode.NodeType nodeType = node.getNodeType();
				if (DBNormalizerTreeNode.NodeType.TABLE.equals(nodeType)
						|| DBNormalizerTreeNode.NodeType.TABLE_CONTAINER.equals(nodeType)) {
					Object o = node.getUserObject();
					if (DBNormalizerTreeNode.NodeType.TABLE.equals(nodeType)) {
						Table incompleteTable = (Table) o;
						try {
							Table completeTable = DatabaseMetadataUtil.getTableInfo(session,
									incompleteTable.getCatalog(), incompleteTable.getSchema(),
									incompleteTable.getName(), true);
							o = completeTable;
						} catch (SQLException ex) {
							// if an exception occurs, we just add the incomplete table info
							// to the selectedItems-list and log an error
							logger.error("Error loading table information for table '"
									+ incompleteTable.getQualifiedName() + "'", ex);
						}
					}
					AbstractDBItem dbItem = (AbstractDBItem) o;
					this.selectedItems.add(dbItem);
				}
			}
			accepted = true;
		} else if (e.getSource() == cancelButton) {
			accepted = false;
		}
		setVisible(false);
	}

	public DBNormalizerSession getSession() {
		return session;
	}

	public void setSession(DBNormalizerSession session) {
		this.dbTree.setSession(session);
		this.session = session;
	}

	public List<AbstractDBItem> getSelectedItems() {
		return selectedItems;
	}

	public boolean isAccepted() {
		return accepted;
	}
}
