package de.fesenmeyer.dbnormalizer.gui;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;

import de.fesenmeyer.dbnormalizer.core.DBNormalizerSession;
import de.fesenmeyer.dbnormalizer.core.FD;
import de.fesenmeyer.dbnormalizer.core.FDAlphabeticalComparator;
import de.fesenmeyer.dbnormalizer.core.FDCheckState;
import de.fesenmeyer.dbnormalizer.core.NormalForm;
import de.fesenmeyer.dbnormalizer.core.Table;
import de.fesenmeyer.dbnormalizer.core.algorithms.FDChecker;
import de.fesenmeyer.dbnormalizer.core.algorithms.NFDeterminationResult;
import de.fesenmeyer.dbnormalizer.core.dbaccess.DBException;
import de.fesenmeyer.dbnormalizer.core.dbaccess.DataSet;
import de.fesenmeyer.dbnormalizer.core.log.DBNormalizerLogger;
import de.fesenmeyer.dbnormalizer.core.util.MiscUtil;
import de.fesenmeyer.dbnormalizer.gui.util.JComponentCellEditor;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class FDAnalysisPanel extends JPanel {

	private static DBNormalizerLogger logger = DBNormalizerLogger.getLogger(FDAnalysisPanel.class);

	private DBNormalizerSession session = null;

	private JTable fdAnalysisTable = null;

	private FDAnalysisTableModel fdAnalysisTableModel = null;

	private JButton checkFDsButton = null;

	private JButton useAsEditFDsButton;

	private TableTabbedPanel tableTabbedPanel;

	public FDAnalysisPanel() {
		super();

		this.fdAnalysisTable = new JTable();
		this.fdAnalysisTableModel = new FDAnalysisTableModel();

		this.fdAnalysisTable.setModel(this.fdAnalysisTableModel);
		this.fdAnalysisTable.setSurrendersFocusOnKeystroke(true);
		this.fdAnalysisTable.getTableHeader().setReorderingAllowed(false);

		this.fdAnalysisTable.getColumnModel().getColumn(FDAnalysisTableModel.CHECK_INFO_INDEX)
				.setCellRenderer(new TableCellRenderer() {
					public Component getTableCellRendererComponent(JTable table, Object value,
							boolean isSelected, boolean hasFocus, int row, int column) {
						return (JComponent) value;
					}
				});

		this.fdAnalysisTable.getColumnModel().getColumn(FDAnalysisTableModel.CHECK_INFO_INDEX)
				.setCellEditor(new JComponentCellEditor());

		BoxLayout fdPanelLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(fdPanelLayout);

		JScrollPane tableScrollPane = new JScrollPane(this.fdAnalysisTable);
		this.add(tableScrollPane);

		JPanel fdButtonPanel = new JPanel();

		this.checkFDsButton = new JButton("Check FDs");
		this.checkFDsButton.setEnabled(false);
		this.checkFDsButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				checkFDButtonPressed(e);
			}

		});

		fdButtonPanel.add(this.checkFDsButton);

		this.useAsEditFDsButton = new JButton("Use MinCover to edit");
		this.useAsEditFDsButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				useAsEditFDsButtonPressed(e);
			}

		});

		fdButtonPanel.add(this.useAsEditFDsButton);

		this.add(fdButtonPanel);
	}

	public void setTable(Table table) {
		this.fdAnalysisTableModel.setTable(table);
	}

	public void refresh() {
		this.fdAnalysisTableModel.refreshTable();
	}

	private void checkFDButtonPressed(ActionEvent e) {
		try {
			this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			
			List<FD> fds = fdAnalysisTableModel.getFdList();
			int toIdx = fds.size();
	
			Connection conn = session.getConnection();
	
			FDChecker fdChecker = new FDChecker(conn);
	
			for (int i = 0; i < toIdx; i++) {
				FD fd = fds.get(i);
				String tableName = fdAnalysisTableModel.getTable().getQualifiedName();
				FDCheckState fdCheckState = null;
				try {
					fdCheckState = fdChecker.checkFD(tableName, fd);
					fd.setFdCheckState(fdCheckState);
				} catch (DBException ex) {
					logger.error("Error checking FD '" + fd.toString() + "'", ex);
				}
			}
			fdAnalysisTableModel.fireTableDataChanged();
		} catch (Exception ex) {
			logger.error("An unexpected exception occurred", ex);
		} finally {
			this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}

	private void useAsEditFDsButtonPressed(ActionEvent e) {
		Table table = fdAnalysisTableModel.getTable();
		Set<FD> minCover = table.getFDMinCover();
		table.setFDs(minCover);
		tableTabbedPanel.getFdModPanel().refresh();
	}

	private final class FDAnalysisTableModel extends AbstractTableModel {
		public static final int LHS_INDEX = 0;

		public static final int RHS_INDEX = 1;

		public static final int CHECK_INDEX = 2;

		public static final int CHECK_INFO_INDEX = 3;

		public static final int NF_VIOLATION_INDEX = 4;

		private Table table = null;

		private List<FD> fdList = new ArrayList<FD>();

		private Map<FD, NormalForm> violatingFDs = new TreeMap<FD, NormalForm>();

		public FDAnalysisTableModel() {

		}

		public FDAnalysisTableModel(Table table) {
			setTable(table);
		}

		public int getColumnCount() {
			return 5;
		}

		public int getRowCount() {
			return fdList.size();
		}

		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			boolean editable = false;
			switch (columnIndex) {
			case CHECK_INFO_INDEX:
				editable = true;
				break;
			default:
				break;
			}
			return editable;
		}

		public Object getValueAt(final int rowIndex, final int colIndex) {
			Object value = null;

			final FD fd = fdList.get(rowIndex);
			switch (colIndex) {
			case LHS_INDEX:
				value = MiscUtil.getCollectionAsString(fd.getLHS());
				break;
			case RHS_INDEX:
				value = MiscUtil.getCollectionAsString(fd.getRHS());
				break;
			case CHECK_INDEX:
				value = fd.getFdCheckState().toString();
				break;
			case CHECK_INFO_INDEX:
				JButton fdCheckInfoButton = new JButton("...");
				if (FDCheckState.NOT_SATISFIED.equals(fd.getFdCheckState())) {
					fdCheckInfoButton.setEnabled(true);
				} else {
					fdCheckInfoButton.setEnabled(false);
				}
				fdCheckInfoButton.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent e) {
						if (fdAnalysisTable.getCellEditor() != null) {
							fdAnalysisTable.getCellEditor().stopCellEditing();
						}
						// load the not satisfiedTupels
						FDChecker fdChecker = new FDChecker(session.getConnection());

						try {
							DataSet dataSet = fdChecker.getNotSatisfiedTupels(table
									.getQualifiedName(), fd);
							FDCheckInfoFrame fdCheckInfoFrame = new FDCheckInfoFrame(fd, dataSet);
							fdCheckInfoFrame.setVisible(true);
						} catch (DBException ex) {
							logger.error("Unexpected DB Exception while retrieving "
									+ "not satisfied tupels for FD '" + fd.toString() + "'", ex);
						}

					}

				});
				value = fdCheckInfoButton;
				break;
			case NF_VIOLATION_INDEX:
				FD currentFD = fdList.get(rowIndex);
				NormalForm violatedNF = violatingFDs.get(currentFD);
				if (violatedNF == null) {
					value = "";
				} else {
					value = violatedNF.toString();
				}
				break;
			default:
				logger.error("Trying to get value for undefined column");
				break;
			}
			return value;
		}

		@Override
		public String getColumnName(int column) {
			String columnName = "";
			switch (column) {
			case LHS_INDEX:
				columnName = "LHS";
				break;
			case RHS_INDEX:
				columnName = "RHS";
				break;
			case CHECK_INDEX:
				columnName = "satisfied";
				break;
			case NF_VIOLATION_INDEX:
				columnName = "violated NF";
				break;
			}

			return columnName;
		}

		public Table getTable() {
			return table;
		}

		public void setTable(Table table) {
			this.table = table;
			refreshTable();
		}

		public void refreshTable() {
			this.fdList = new ArrayList<FD>(table.getFDMinCover());
			Collections.sort(this.fdList, new FDAlphabeticalComparator());
			NFDeterminationResult nfDetermResult = table.getNfDeterminationResult();
			setNFDeterminationResult(nfDetermResult);
			fireTableDataChanged();
		}

		public List<FD> getFdList() {
			return fdList;
		}

		private void setNFDeterminationResult(NFDeterminationResult result) {
			Map<NormalForm, Set<FD>> violatingFDsByNF = result.getViolatingFDs();
			Map<FD, NormalForm> violatingFDs = new TreeMap<FD, NormalForm>(new FDAlphabeticalComparator());
			for (NormalForm nf : violatingFDsByNF.keySet()) {
				for (FD fd : violatingFDsByNF.get(nf)) {
					violatingFDs.put(fd, nf);
				}
			}
			this.violatingFDs = violatingFDs;
		}

	}

	public DBNormalizerSession getSession() {
		return session;
	}

	public void setSession(DBNormalizerSession session) {
		this.session = session;
		this.checkFDsButton.setEnabled(session.getDbConfig() != null);
	}

	public FDAnalysisTableModel getFdAnalysisTableModel() {
		return fdAnalysisTableModel;
	}

	public void setTableTabbedPanel(TableTabbedPanel tableTabbedPanel) {
		this.tableTabbedPanel = tableTabbedPanel;
	}

}
