package de.fesenmeyer.dbnormalizer.gui;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.TreeSet;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;

import de.fesenmeyer.dbnormalizer.core.AttributeStringSet;
import de.fesenmeyer.dbnormalizer.core.DBNormalizerException;
import de.fesenmeyer.dbnormalizer.core.DBNormalizerSession;
import de.fesenmeyer.dbnormalizer.core.FD;
import de.fesenmeyer.dbnormalizer.core.FDAlphabeticalComparator;
import de.fesenmeyer.dbnormalizer.core.FDCheckState;
import de.fesenmeyer.dbnormalizer.core.Table;
import de.fesenmeyer.dbnormalizer.core.algorithms.FDChecker;
import de.fesenmeyer.dbnormalizer.core.dbaccess.DBException;
import de.fesenmeyer.dbnormalizer.core.dbaccess.DataSet;
import de.fesenmeyer.dbnormalizer.core.log.DBNormalizerLogger;
import de.fesenmeyer.dbnormalizer.core.util.MiscUtil;
import de.fesenmeyer.dbnormalizer.gui.util.JComponentCellEditor;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class FDModificationPanel extends JPanel {

	private static DBNormalizerLogger logger = DBNormalizerLogger
			.getLogger(FDModificationPanel.class);

	private DBNormalizerSession session = null;

	private JTable fdModTable = null;

	private FDModTableModel fdModTableModel = null;

	private JButton checkFDsButton = null;

	private JButton saveFDsButton;

	private TableMainPanel tableMainPanel;

	public FDModificationPanel() {
		super();

		this.fdModTable = new JTable();
		this.fdModTableModel = new FDModTableModel();

		this.fdModTable.setModel(this.fdModTableModel);
		this.fdModTable.setSurrendersFocusOnKeystroke(true);
		this.fdModTable.getTableHeader().setReorderingAllowed(false);

		this.fdModTable.getColumnModel().getColumn(FDModTableModel.CHECK_INFO_INDEX)
				.setCellRenderer(new TableCellRenderer() {
					public Component getTableCellRendererComponent(JTable table, Object value,
							boolean isSelected, boolean hasFocus, int row, int column) {
						return (JComponent) value;
					}
				});

		// set editor for JButton column
		this.fdModTable.getColumnModel().getColumn(FDModTableModel.CHECK_INFO_INDEX).setCellEditor(
				new JComponentCellEditor());

		BoxLayout fdPanelLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(fdPanelLayout);

		JScrollPane tableScrollPane = new JScrollPane(this.fdModTable);
		this.add(tableScrollPane);

		JPanel fdButtonPanel = new JPanel();

		JButton removeRowButton = new JButton("Remove");
		removeRowButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				int[] indices = fdModTable.getSelectedRows();
				List<FD> fdList = fdModTableModel.getFdList();

				for (int i = 0; i < indices.length; i++) {
					int idx = indices[i];
					fdModTableModel.getTable().getFDs().remove(fdList.get(idx));
				}
				fdModTableModel.refreshTable();
			}

		});
		fdButtonPanel.add(removeRowButton);

		JButton addRowButton = new JButton("Add");
		addRowButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				FD fd = new FD();
				List<FD> fdList = fdModTableModel.getFdList();
				fdList.add(fd);
				fdModTableModel.fireTableRowsInserted(fdList.size() - 1, fdList.size() - 1);
			}

		});
		fdButtonPanel.add(addRowButton);

		saveFDsButton = new JButton("Save");
		saveFDsButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				// it's important to stop cell editing that
				// no values get lost
				if (fdModTable.getCellEditor() != null) {
					fdModTable.getCellEditor().stopCellEditing();
				}

				fdModTableModel.acceptFDs();
				fdsChanged();
			}

		});
		fdButtonPanel.add(saveFDsButton);

		this.checkFDsButton = new JButton("Check FDs");
		this.checkFDsButton.setEnabled(false);
		this.checkFDsButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				checkFDButtonPressed(e);
			}

		});

		fdButtonPanel.add(this.checkFDsButton);

		this.add(fdButtonPanel);
	}

	private void fdsChanged() {
		this.tableMainPanel.fdsChanged();
	}

	public void setTable(Table table) {
		this.fdModTableModel.setTable(table);
	}

	private void checkFDButtonPressed(ActionEvent e) {
		try {
			this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			// it's important to stop cell editing that
			// that no incompletely specified FDs get checked
			if (fdModTable.getCellEditor() != null) {
				fdModTable.getCellEditor().stopCellEditing();
			}
			
			List<FD> fds = fdModTableModel.getFdList();
			int toIdx = fds.size();
	
			Connection conn = session.getConnection();
	
			FDChecker fdChecker = new FDChecker(conn);
	
			for (int i = 0; i < toIdx; i++) {
				FD fd = fds.get(i);
				String tableName = fdModTableModel.getTable().getQualifiedName();
				FDCheckState fdCheckState = null;
				try {
					fdCheckState = fdChecker.checkFD(tableName, fd);
					fd.setFdCheckState(fdCheckState);
				} catch (DBException ex) {
					logger.error("Error checking FD '" + fd.toString() + "'", ex);
				}
			}
	
			fdModTableModel.fireTableDataChanged();
		} catch (Exception ex) {
			logger.error("An unexpected exception occurred", ex);
		} finally {
			this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}

	@SuppressWarnings("serial")
	private final class FDModTableModel extends AbstractTableModel {
		public static final int LHS_INDEX = 0;

		public static final int RHS_INDEX = 1;

		public static final int CHECK_INDEX = 2;

		public static final int CHECK_INFO_INDEX = 3;

		Table table = null;

		List<FD> fdList = new ArrayList<FD>();

		public FDModTableModel() {
		}

		public FDModTableModel(Table table) {
			setTable(table);
		}

		public int getColumnCount() {
			return 4;
		}

		public int getRowCount() {
			return fdList.size();
		}

		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			boolean editable = false;
			switch (columnIndex) {
			case LHS_INDEX:
			case RHS_INDEX:
			case CHECK_INFO_INDEX:
				editable = true;
				break;
			default:
				break;
			}
			return editable;
		}

		public Object getValueAt(final int rowIndex, final int colIndex) {
			Object value = null;

			final FD fd = fdList.get(rowIndex);
			switch (colIndex) {
			case LHS_INDEX:
				value = MiscUtil.getCollectionAsString(fd.getLHS());
				break;
			case RHS_INDEX:
				value = MiscUtil.getCollectionAsString(fd.getRHS());
				break;
			case CHECK_INDEX:
				value = fd.getFdCheckState().toString();
				break;
			case CHECK_INFO_INDEX:
				JButton fdCheckInfoButton = new JButton("...");
				if (FDCheckState.NOT_SATISFIED.equals(fd.getFdCheckState())) {
					fdCheckInfoButton.setEnabled(true);
				} else {
					fdCheckInfoButton.setEnabled(false);
				}
				fdCheckInfoButton.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent e) {
						if (fdModTable.getCellEditor() != null) {
							fdModTable.getCellEditor().stopCellEditing();
						}
						// load the not satisfiedTupels
						FDChecker fdChecker = new FDChecker(session.getConnection());

						try {
							DataSet dataSet = fdChecker.getNotSatisfiedTupels(table
									.getQualifiedName(), fd);
							FDCheckInfoFrame fdCheckInfoFrame = new FDCheckInfoFrame(fd, dataSet);
							fdCheckInfoFrame.setVisible(true);
						} catch (DBException ex) {
							logger.error("Unexpected DB Exception while retrieving "
									+ "not satisfied tupels for FD '" + fd.toString() + "'", ex);
						}

					}

				});
				value = fdCheckInfoButton;
				break;
			default:
				logger.error("Trying to get value for undefined column");
				break;
			}
			return value;
		}

		@Override
		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			String attributeSetString = null;
			FD fd = null;

			switch (columnIndex) {
			case LHS_INDEX:
				// parse the String value
				attributeSetString = (String) aValue;
				fd = fdList.get(rowIndex);
				AttributeStringSet lhs = null;
				try {
					lhs = new AttributeStringSet(MiscUtil.parseAttributeStringSet(
							attributeSetString, table));
				} catch (DBNormalizerException e) {
					// TODO other solution?
					throw new RuntimeException(e);
				}
				fd.setLHS(lhs);
				break;
			case RHS_INDEX:
				// parse the String value
				attributeSetString = (String) aValue;
				fd = fdList.get(rowIndex);
				AttributeStringSet rhs = null;
				try {
					rhs = new AttributeStringSet(MiscUtil.parseAttributeStringSet(
							attributeSetString, table));
				} catch (DBNormalizerException e) {
					// TODO other solution?
					throw new RuntimeException(e);
				}
				fd.setRHS(rhs);
				break;
			case CHECK_INFO_INDEX:
				// do nothing
				break;
			default:
				logger.error("Trying to set value for undefined column");
				break;
			}

		}

		@Override
		public String getColumnName(int column) {
			String columnName = "";
			switch (column) {
			case LHS_INDEX:
				columnName = "LHS";
				break;
			case RHS_INDEX:
				columnName = "RHS";
				break;
			case CHECK_INDEX:
				columnName = "satisfied";
				break;
			}
			return columnName;
		}

		/**
		 * saves the FDs from the table to the internal table representation
		 */
		public void acceptFDs() {
			int idx = fdList.size();
			TreeSet<FD> fds = new TreeSet<FD>();
			for (int i = 0; i < idx; i++) {
				fds.add(fdList.get(i));
			}
			table.setFDs(fds);
		}

		public Table getTable() {
			return table;
		}

		public void setTable(Table table) {
			this.table = table;
			refreshTable();
		}

		public void refreshTable() {
			this.fdList = new ArrayList<FD>(table.getFDs());
			Collections.sort(this.fdList, new FDAlphabeticalComparator());
			fireTableDataChanged();
		}

		public List<FD> getFdList() {
			return fdList;
		}

	}

	public DBNormalizerSession getSession() {
		return session;
	}

	public void setSession(DBNormalizerSession session) {
		this.session = session;
		this.checkFDsButton.setEnabled(session.getDbConfig() != null);
	}

	public void refresh() {
		this.fdModTableModel.refreshTable();
	}

	public void setTableMainPanel(TableMainPanel tableMainPanel) {
		this.tableMainPanel = tableMainPanel;
	}
}
