package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import de.fesenmeyer.dbnormalizer.core.FK;
import de.fesenmeyer.dbnormalizer.core.TableContainer;
import de.fesenmeyer.dbnormalizer.core.util.MiscUtil;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class FKInfoPanel extends JPanel {

	private String title = null;

	private JTable fkInfoTable = null;

	private FKInfoTableModel fkInfoTableModel;

	public FKInfoPanel(String title) {
		super(new BorderLayout());
		this.title = title;

		this.setBorder(BorderFactory.createTitledBorder(this.title));

		this.fkInfoTableModel = new FKInfoTableModel();
		this.fkInfoTable = new JTable(this.fkInfoTableModel);

		JScrollPane tableScrollPane = new JScrollPane(this.fkInfoTable);
		this.add(tableScrollPane, BorderLayout.CENTER);
	}

	public void setFks(List<FK> fks, TableContainer tableContainer) {
		this.fkInfoTableModel.setFks(fks, tableContainer);
	}

	public class FKInfoTableModel extends AbstractTableModel {

		private List<FK> fks = new ArrayList<FK>();

		private String[] columnNames = { "FKTable", "FKCols", "FKName", "PKTable", "PKCols",
				"PKName", "UpdateRule", "DeleteRule" };

		private TableContainer tableContainer = null;

		public int getColumnCount() {
			return columnNames.length;
		}

		@Override
		public String getColumnName(int column) {
			return this.columnNames[column];
		}

		public int getRowCount() {
			return fks.size();
		}

		public Object getValueAt(int rowIndex, int colIndex) {
			Object value = null;

			FK fk = fks.get(rowIndex);
			switch (colIndex) {
			case 0:
				value = createCustomTableName(fk.getFkTableName(), fk.getFkCatalog(), fk
						.getFkSchema());
				break;
			case 1:
				value = MiscUtil.getCollectionAsString(fk.getFkColumnNames());
				break;
			case 2:
				value = fk.getName();
				break;
			case 3:
				value = createCustomTableName(fk.getPkTableName(), fk.getPkCatalog(), fk
						.getPkSchema());
				break;
			case 4:
				value = MiscUtil.getCollectionAsString(fk.getPkColumnNames());
				break;
			case 5:
				value = fk.getPkName();
				break;
			case 6:
				value = MiscUtil.getDeleteOrUpdateRuleAsString(fk.getUpdateRule());
				break;
			case 7:
				value = MiscUtil.getDeleteOrUpdateRuleAsString(fk.getDeleteRule());
				break;
			}
			return value;
		}

		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return false;
		}

		public List<FK> getFks() {
			return fks;
		}

		public void setFks(List<FK> fks, TableContainer tableContainer) {
			this.fks = fks;
			this.tableContainer  = tableContainer;
			fireTableDataChanged();
		}
		
		private String createCustomTableName(String tableName, String catalog, String schema) {
			String str = null;
			TableContainer otherContainer = MiscUtil.createTableContainer(catalog, schema);
			if (otherContainer.equals(this.tableContainer)) {
				str = tableName;
			} else {
				str = MiscUtil.getQualifiedTableName(tableName, catalog, schema);
			}
			return str;
		}

	}

}
