package de.fesenmeyer.dbnormalizer.gui;

import java.awt.Image;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;

import javax.swing.ImageIcon;

import de.fesenmeyer.dbnormalizer.core.log.DBNormalizerLogger;

/**
 * @author DF
 *
 */
public final class IconManager {

	private static DBNormalizerLogger logger = DBNormalizerLogger.getLogger(IconManager.class);

	public static final String DBNORMALIZER_ICON = "dbnormalizer.gif";

	public static final String TABLE_ICON = "table.gif";

	public static final String COLUMN_ICON = "columns.gif";
	
	public static final String FOLDER_ICON = "folder.gif";

	private static IconManager INSTANCE = null;

	HashMap<String, ImageIcon> imageIcons = new HashMap<String, ImageIcon>();

	private IconManager() {

	}

	public static IconManager getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new IconManager();
		}
		return INSTANCE;
	}

	/**
	 * gets the image icon with the given name.
	 * 
	 * @param name
	 *            one of the constants defined in this class, for example
	 *            {@link IconManager#DBNORMALIZER_ICON}.
	 * @return returns the icon, if it could be created without error; returns <code>null</code>,
	 *         otherwise.
	 */
	public ImageIcon getIcon(String name) {
		ImageIcon icon = null;
		if (imageIcons.containsKey(name)) {
			icon = imageIcons.get(name);
		} else {
			icon = createImageIcon("icons/" + name, null);
			imageIcons.put(name, icon);
		}
		return icon;
	}

	public Image getImage(String name) {
		ImageIcon icon = getIcon(name);
		return icon.getImage();
	}

	/**
	 * helper method to create an image icon.
	 * 
	 * @param path
	 *            the path of the icon (relative to the icons folder).
	 * @param description
	 *            the description of the icon
	 * @return returns the image icon, if no error occurs; returns <code>null</code>, otherwise.
	 */
	private ImageIcon createImageIcon(String path, String description) {
		ImageIcon imgIcon = null;

		// we use an InputStream, not a URL, to build the ImageIcon
		// to allow the images to be loaded from a JAR file
		InputStream in = getClass().getResourceAsStream(path);

		if (in == null) {
			logger.error("Couldn't find file: '" + path + "'");
		} else {
			ByteArrayOutputStream out = new ByteArrayOutputStream();

			byte buf[] = new byte[1024];
			int len;
			try {
				while ((len = in.read(buf)) > 0) {
					out.write(buf, 0, len);
				}
				imgIcon = new ImageIcon(out.toByteArray(), description);
			} catch (IOException e) {
				logger.error("Error when loading image icon '" + description + "' from path '"
						+ path + "'", e);
			} finally {
				try {
					in.close();
				} catch (IOException e) {
					// do nothing
				}
			}
		}

		return imgIcon;
	}

}
