package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

import de.fesenmeyer.dbnormalizer.core.FK;
import de.fesenmeyer.dbnormalizer.core.NormalForm;
import de.fesenmeyer.dbnormalizer.core.TableContainer;
import de.fesenmeyer.dbnormalizer.core.algorithms.NormalizationFKsProposal;
import de.fesenmeyer.dbnormalizer.core.algorithms.NormalizationProposal;
import de.fesenmeyer.dbnormalizer.core.log.DBNormalizerLogger;
import de.fesenmeyer.dbnormalizer.gui.util.DBNormalizerColors;
import de.fesenmeyer.dbnormalizer.gui.util.FileFilters;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class NormPropPanel extends JPanel {

	private static DBNormalizerLogger logger = DBNormalizerLogger.getLogger(NormPropPanel.class);

	private JFileChooser saveFileChooser = null;

	private NormalizationProposal normProposal = null;

	private JLabel tableNameLbl = null;
	private JLabel nfLbl = null;

	private JPanel fkPanel = null;
	private FKInfoPanel innerFKsPanel = null;
	private FKInfoPanel importedFKsPanel = null;
	private FKInfoPanel exportedFKsPanel = null;
	private FKInfoPanel hierarchicalFKsPanel = null;

	private JPanel scriptPanel = null;
	private JTextArea scriptTextArea = null;

	private JButton saveScriptButton = null;

	public NormPropPanel() {
		super(new BorderLayout());

		JPanel nfInfoPanel = new JPanel(new GridBagLayout());
		nfInfoPanel.setBorder(BorderFactory.createTitledBorder("Normalform"));
		JLabel tblNameCaptionLbl = new JLabel("Normalization Proposal for table:");
		tblNameCaptionLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		this.tableNameLbl = new JLabel("");
		JLabel nfCaptionLbl = new JLabel("Normal Form:");
		nfCaptionLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		this.nfLbl = new JLabel("?");
		this.nfLbl.setOpaque(true);
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 0;
		nfInfoPanel.add(tblNameCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		nfInfoPanel.add(this.tableNameLbl, c);
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		c.weightx = 0;
		nfInfoPanel.add(nfCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		nfInfoPanel.add(this.nfLbl, c);

		this.add(nfInfoPanel, BorderLayout.PAGE_START);

		JTabbedPane tabbedPane = new JTabbedPane();

		// FK Panel
		this.fkPanel = new JPanel();
		LayoutManager fkPanelLayout = new BoxLayout(this.fkPanel, BoxLayout.Y_AXIS);
		this.fkPanel.setLayout(fkPanelLayout);

		this.innerFKsPanel = new FKInfoPanel("FKs to be created between the new tables");
		this.fkPanel.add(this.innerFKsPanel);

		this.importedFKsPanel = new FKInfoPanel(
				"FKs to be created due to imported keys from other tables");
		this.fkPanel.add(this.importedFKsPanel);

		this.exportedFKsPanel = new FKInfoPanel(
				"FKs to be created due to exported keys to other tables");
		this.fkPanel.add(this.exportedFKsPanel);

		this.hierarchicalFKsPanel = new FKInfoPanel(
				"FKs to be created because of hierarchical FKs in old table");
		this.fkPanel.add(this.hierarchicalFKsPanel);

		tabbedPane.add("FK Information", this.fkPanel);

		// Script Panel
		this.scriptPanel = new JPanel();
		LayoutManager scriptPanelLayout = new BorderLayout();
		this.scriptPanel.setLayout(scriptPanelLayout);

		this.scriptTextArea = new JTextArea();
		this.scriptTextArea.setEditable(false);
		JScrollPane scriptScrollPane = new JScrollPane(this.scriptTextArea);

		this.scriptPanel.add(scriptScrollPane, BorderLayout.CENTER);

		this.saveScriptButton = new JButton("Save");
		this.saveScriptButton.setEnabled(false);
		this.saveScriptButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				saveScript();
			}

		});
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		buttonPanel.add(saveScriptButton);

		this.scriptPanel.add(buttonPanel, BorderLayout.PAGE_END);

		tabbedPane.add("SQL Normalization Script", this.scriptPanel);

		this.add(tabbedPane, BorderLayout.CENTER);
	}

	public void setNormProposal(NormalizationProposal normProposal) {
		this.normProposal = normProposal;

		this.tableNameLbl.setText(this.normProposal.getOldTable().getName());
		setNF(this.normProposal.getNf());
		
		TableContainer tableContainer = normProposal.getOldTable().getContainer();

		NormalizationFKsProposal fksProposal = this.normProposal.getFksProposal();
		this.innerFKsPanel.setFks(new ArrayList<FK>(fksProposal.getInnerFKS()),
				tableContainer);
		this.importedFKsPanel.setFks(new ArrayList<FK>(fksProposal.getImportedFKs()),
				tableContainer);
		this.exportedFKsPanel.setFks(new ArrayList<FK>(fksProposal.getExportedFKs()),
				tableContainer);
		this.hierarchicalFKsPanel.setFks(new ArrayList<FK>(fksProposal.getHierarchicalFKs()),
				tableContainer);

		// a script has only been generated, if the original table
		// is a DB-Table
		if (normProposal.getOldTable().isDbTable()) {
			this.saveScriptButton.setEnabled(true);
			this.scriptTextArea.setText(normProposal.getSqlScript());
		} else {
			this.saveScriptButton.setEnabled(false);
			this.scriptTextArea.setText(null);
		}
	}

	private void setNF(NormalForm nf) {
		this.nfLbl.setText(nf.toString());
		Color color = null;
		switch (nf) {
		case BCNF:
			color = DBNormalizerColors.COLOR_OK;
			break;
		case NF3:
			color = DBNormalizerColors.COLOR_WARN;
			break;
		default:
			color = DBNormalizerColors.COLOR_ERROR;
			break;
		}
		this.nfLbl.setBackground(color);
	}

	private JFileChooser getSaveFileChooser() {
		if (this.saveFileChooser == null) {
			this.saveFileChooser = new JFileChooser();
			this.saveFileChooser.setFileFilter(FileFilters.getSQLFileFilter());
		}
		return this.saveFileChooser;
	}

	private void saveScript() {
		int retVal = getSaveFileChooser().showSaveDialog(this.getTopLevelAncestor());
		if (retVal == JFileChooser.APPROVE_OPTION) {
			File f = getSaveFileChooser().getSelectedFile();
			FileWriter fw = null;
			try {
				fw = new FileWriter(f);
				fw.write(this.normProposal.getSqlScript());
			} catch (IOException e) {
				logger.error(
						"Error while trying to save sql script to file " + f.getAbsolutePath(), e);
			} finally {
				if (fw != null) {
					try {
						fw.close();
					} catch (IOException e) {
						logger.error("Error closing FileWriter", e);
					}
				}
			}

		}
	}

}
