package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import de.fesenmeyer.dbnormalizer.core.CandKey;
import de.fesenmeyer.dbnormalizer.core.NormalForm;
import de.fesenmeyer.dbnormalizer.core.PK;
import de.fesenmeyer.dbnormalizer.core.Table;
import de.fesenmeyer.dbnormalizer.core.algorithms.NFDeterminationResult;
import de.fesenmeyer.dbnormalizer.gui.util.DBNormalizerColors;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class NormPropTablePanel extends JPanel {

	private JLabel tableNameLbl = null;

	private JLabel nfLbl = null;

	private SimpleFDPanel simpleFDPanel = null;

	private TableInformationPanel tableInfoPanel = null;

	private JList candKeyList = null;

	private JList pkList = null;

	public NormPropTablePanel() {
		super();

		BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(layout);

		// NF Info Panel
		JPanel nfInfoPanel = new JPanel(new GridBagLayout());
		nfInfoPanel.setBorder(BorderFactory.createTitledBorder("Tablename and Normalform"));
		JLabel tblNameCaptionLbl = new JLabel("Table name");
		this.tableNameLbl = new JLabel("");
		JLabel nfCaptionLbl = new JLabel("Normal Form");
		this.nfLbl = new JLabel("?");
		this.nfLbl.setOpaque(true);
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 0;
		nfInfoPanel.add(tblNameCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		nfInfoPanel.add(this.tableNameLbl, c);
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		c.weightx = 0;
		nfInfoPanel.add(nfCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		nfInfoPanel.add(this.nfLbl, c);

		this.add(nfInfoPanel);

		// FD and table info Panel
		JTabbedPane fdAndTableInfoPanel = new JTabbedPane();
		fdAndTableInfoPanel.setBorder(BorderFactory.createTitledBorder("FDs and Table Info"));

		this.simpleFDPanel = new SimpleFDPanel();
		fdAndTableInfoPanel.add("FDs", this.simpleFDPanel);

		this.tableInfoPanel = new TableInformationPanel();
		fdAndTableInfoPanel.add("Table Info", this.tableInfoPanel);

		this.add(fdAndTableInfoPanel);

		// Keys panel
		JPanel keyPanel = new JPanel(new GridBagLayout());
		c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);
		keyPanel.setBorder(BorderFactory.createTitledBorder("Keys"));

		this.candKeyList = new JList(new DefaultListModel());
		JScrollPane candKeyScrollPane = new JScrollPane(this.candKeyList);
		JPanel candKeyPane = new JPanel(new BorderLayout());
		candKeyPane.setBorder(BorderFactory.createTitledBorder("Candidate Keys"));
		candKeyPane.add(candKeyScrollPane, BorderLayout.CENTER);

		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0.5;
		c.weighty = 1.0;
		c.fill = GridBagConstraints.BOTH;
		keyPanel.add(candKeyPane, c);

		this.pkList = new JList(new DefaultListModel());
		JScrollPane pkScrollPane = new JScrollPane(this.pkList);
		JPanel pkPane = new JPanel(new BorderLayout());
		pkPane.setBorder(BorderFactory.createTitledBorder("PK"));
		pkPane.add(pkScrollPane, BorderLayout.CENTER);

		c.gridx = 1;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0.5;
		c.weighty = 1.0;
		c.fill = GridBagConstraints.BOTH;
		keyPanel.add(pkPane, c);

		this.add(keyPanel);
	}

	private void setNF(NormalForm nf) {
		this.nfLbl.setText(nf.toString());
		Color color = null;
		switch (nf) {
		case BCNF:
			color = DBNormalizerColors.COLOR_OK;
			break;
		case NF3:
			color = DBNormalizerColors.COLOR_WARN;
			break;
		default:
			color = DBNormalizerColors.COLOR_ERROR;
			break;
		}
		this.nfLbl.setBackground(color);
	}

	public void setTable(Table table) {
		NFDeterminationResult nfDeterminationResult = table.getNfDeterminationResult();

		tableNameLbl.setText(table.getName());
		setNF(nfDeterminationResult.getNf());
		setKeysForTable(table);

		this.simpleFDPanel.setTable(table);
		this.tableInfoPanel.setTable(table);
	}

	public void setKeysForTable(Table table) {
		DefaultListModel listModel = (DefaultListModel) candKeyList.getModel();
		listModel.clear();
		for (CandKey candidateKey : table.getCandKeys()) {
			listModel.addElement(candidateKey.getAttributes());
		}

		PK pk = table.getPk();
		if (pk != null && pk.getAttributes().size() > 0) {
			DefaultListModel pklistModel = (DefaultListModel) pkList.getModel();
			pklistModel.clear();
			for (String attr : pk.getAttributes()) {
				pklistModel.addElement(attr);
			}
		}

	}
}
