package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import de.fesenmeyer.dbnormalizer.core.FD;
import de.fesenmeyer.dbnormalizer.core.FDAlphabeticalComparator;
import de.fesenmeyer.dbnormalizer.core.NormalForm;
import de.fesenmeyer.dbnormalizer.core.Table;
import de.fesenmeyer.dbnormalizer.core.algorithms.NFDeterminationResult;
import de.fesenmeyer.dbnormalizer.core.util.MiscUtil;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class SimpleFDPanel extends JPanel {

	private JTable simpleFDTable = null;

	private SimpleFDTableModel simpleFDTableModel = null;

	public SimpleFDPanel() {
		super(new BorderLayout());

		this.simpleFDTableModel = new SimpleFDTableModel();

		this.simpleFDTable = new JTable(this.simpleFDTableModel);
		this.simpleFDTable.getTableHeader().setReorderingAllowed(false);

		JScrollPane tableScrollPane = new JScrollPane(this.simpleFDTable);
		this.add(tableScrollPane, BorderLayout.CENTER);
	}

	public void setTable(Table table) {
		this.simpleFDTableModel.setTable(table);
	}

	private final class SimpleFDTableModel extends AbstractTableModel {

		private List<FD> fdList = new ArrayList<FD>();

		private String[] columnNames = new String[] { "LHS", "RHS", "violated NF" };

		private Map<FD, NormalForm> violatingFDs = new TreeMap<FD, NormalForm>();

		public int getColumnCount() {
			return columnNames.length;
		}

		@Override
		public String getColumnName(int column) {
			return columnNames[column];
		}

		public int getRowCount() {
			return fdList.size();
		}

		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return false;
		}

		public Object getValueAt(int rowIndex, int colIndex) {
			Object value = null;

			FD fd = this.fdList.get(rowIndex);

			switch (colIndex) {
			case 0:
				value = MiscUtil.getCollectionAsString(fd.getLHS());
				break;
			case 1:
				value = MiscUtil.getCollectionAsString(fd.getRHS());
				break;
			case 2:
				FD currentFD = fdList.get(rowIndex);
				NormalForm violatedNF = violatingFDs.get(currentFD);
				if (violatedNF == null) {
					value = "";
				} else {
					value = violatedNF.toString();
				}
				break;
			}

			return value;
		}

		public void setTable(Table table) {
			this.fdList = new ArrayList<FD>(table.getFDMinCover());
			Collections.sort(this.fdList, new FDAlphabeticalComparator());
			setNFDeterminationResult(table.getNfDeterminationResult());
			fireTableDataChanged();
		}

		private void setNFDeterminationResult(NFDeterminationResult result) {
			Map<NormalForm, Set<FD>> violatingFDsByNF = result.getViolatingFDs();
			Map<FD, NormalForm> violatingFDs = new TreeMap<FD, NormalForm>(new FDAlphabeticalComparator());
			for (NormalForm nf : violatingFDsByNF.keySet()) {
				for (FD fd : violatingFDsByNF.get(nf)) {
					violatingFDs.put(fd, nf);
				}
			}
			this.violatingFDs = violatingFDs;
		}

	}

}
