package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import de.fesenmeyer.dbnormalizer.core.FK;
import de.fesenmeyer.dbnormalizer.core.PK;
import de.fesenmeyer.dbnormalizer.core.Table;
import de.fesenmeyer.dbnormalizer.core.TableContainer;
import de.fesenmeyer.dbnormalizer.core.util.MiscUtil;
import de.fesenmeyer.dbnormalizer.gui.util.DBNormalizerColors;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class TableInformationPanel extends JPanel {
	private Table table = null;

	private JPanel mainInformationPanel = null;

	private JLabel tableNameLbl = null;

	private JLabel catalogLbl = null;

	private JLabel schemaLbl = null;

	private JLabel pkLbl = null;

	private TableColumnInfoPanel tableColInfoPanel = null;

	private JPanel fkInformationPanel = null;

	private FKInfoPanel importedKeysPanel = null;

	private FKInfoPanel exportedKeysPanel = null;

	public TableInformationPanel() {
		super(new BorderLayout());

		this.mainInformationPanel = new JPanel();
		LayoutManager mainInfoPanelLayout = new BoxLayout(this.mainInformationPanel,
				BoxLayout.Y_AXIS);
		this.mainInformationPanel.setLayout(mainInfoPanelLayout);
		this.tableColInfoPanel = new TableColumnInfoPanel();
		JPanel tableAndPKInfoPanel = new JPanel(new GridBagLayout());
		tableAndPKInfoPanel.setBorder(BorderFactory.createTitledBorder("Table And Key Info"));
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 0;
		// table name caption
		JLabel tableNameCaptionLbl = new JLabel("Table Name:");
		tableNameCaptionLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		tableAndPKInfoPanel.add(tableNameCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		// table name
		this.tableNameLbl = new JLabel("");
		tableAndPKInfoPanel.add(this.tableNameLbl, c);
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		c.weightx = 0;
		// catalog name caption
		JLabel catalogCaptionLbl = new JLabel("Catalog:");
		catalogCaptionLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		tableAndPKInfoPanel.add(catalogCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		// catalog name
		this.catalogLbl = new JLabel("");
		tableAndPKInfoPanel.add(this.catalogLbl, c);
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 1;
		c.weightx = 0;
		// schema name caption
		JLabel schemaCaptionLbl = new JLabel("Schema:");
		schemaCaptionLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		tableAndPKInfoPanel.add(schemaCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		// schema name
		this.schemaLbl = new JLabel("");
		tableAndPKInfoPanel.add(this.schemaLbl, c);
		c.gridx = 0;
		c.gridy = 3;
		c.gridwidth = 1;
		c.weightx = 0;
		// pk caption
		JLabel pkCaptionLabel = new JLabel("PK:");
		pkCaptionLabel.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		tableAndPKInfoPanel.add(pkCaptionLabel, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		// pk
		this.pkLbl = new JLabel("");
		tableAndPKInfoPanel.add(this.pkLbl, c);

		this.mainInformationPanel.add(tableAndPKInfoPanel);
		this.mainInformationPanel.add(this.tableColInfoPanel);

		this.fkInformationPanel = new JPanel();
		LayoutManager fkInfoPanelLayout = new BoxLayout(this.fkInformationPanel, BoxLayout.Y_AXIS);
		this.fkInformationPanel.setLayout(fkInfoPanelLayout);
		this.importedKeysPanel = new FKInfoPanel("Imported Keys");
		this.exportedKeysPanel = new FKInfoPanel("Exported Keys");
		this.fkInformationPanel.add(this.importedKeysPanel);
		this.fkInformationPanel.add(this.exportedKeysPanel);

		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.add("Key and Attribute Info", this.mainInformationPanel);
		tabbedPane.add("FK Info", this.fkInformationPanel);

		this.add(tabbedPane, BorderLayout.CENTER);
	}

	public Table getTable() {
		return table;
	}

	public void setTable(Table table) {
		this.table = table;
		refresh();
	}

	private void refresh() {
		// refresh table info
		this.tableNameLbl.setText(table.getName());
		this.catalogLbl.setText(table.getCatalog());
		this.schemaLbl.setText(table.getSchema());

		// refresh the pk label
		String pkTxt = "";
		PK dbPk = table.getDbPK();
		if (dbPk != null) {
			if (dbPk.getName() != null) {
				pkTxt += dbPk.getName() + " ";
			}
			pkTxt += "(" + MiscUtil.getCollectionAsString(dbPk.getAttributes()) + ")";
		} else {
			pkTxt += "No PK defined";
		}
		this.pkLbl.setText(pkTxt);

		TableContainer tableContainer = table.getContainer();
		
		// refresh each panel
		this.importedKeysPanel.setFks(new ArrayList<FK>(table.getImportedKeys()),
				tableContainer);
		this.exportedKeysPanel.setFks(new ArrayList<FK>(table.getExportedKeys()),
				tableContainer);
		this.tableColInfoPanel.setTableCols(table.getColumnInfos());

	}

}
