package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import de.fesenmeyer.dbnormalizer.core.AttributeStringSet;
import de.fesenmeyer.dbnormalizer.core.CandKey;
import de.fesenmeyer.dbnormalizer.core.DBNormalizerException;
import de.fesenmeyer.dbnormalizer.core.DBNormalizerSession;
import de.fesenmeyer.dbnormalizer.core.FD;
import de.fesenmeyer.dbnormalizer.core.NormalForm;
import de.fesenmeyer.dbnormalizer.core.PK;
import de.fesenmeyer.dbnormalizer.core.Table;
import de.fesenmeyer.dbnormalizer.core.algorithms.FDAlgorithms;
import de.fesenmeyer.dbnormalizer.core.algorithms.NFDetermination;
import de.fesenmeyer.dbnormalizer.core.algorithms.NFDeterminationResult;
import de.fesenmeyer.dbnormalizer.core.algorithms.NormalizationProposal;
import de.fesenmeyer.dbnormalizer.core.algorithms.ThreeNFSynthesisAlgorithmKemper;
import de.fesenmeyer.dbnormalizer.core.log.DBNormalizerLogger;
import de.fesenmeyer.dbnormalizer.core.util.MiscUtil;
import de.fesenmeyer.dbnormalizer.gui.util.DBNormalizerColors;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class TableMainPanel extends JPanel {

	private static DBNormalizerLogger logger = DBNormalizerLogger.getLogger(TableMainPanel.class);

	private DBNormalizerSession session = null;
	private Table table = null;

	private ApplicationFrame applicationFrame = null;

	private JLabel tableNameLbl = null;

	private TableTabbedPanel tableTabbedPanel = null;

	private JButton normProposalButton = null;

	private JLabel keyWarningLbl = null;

	private JList candKeyList = null;

	private JLabel nfWarningLabel = null;

	private JLabel nfLbl = null;

	private JList pkList = null;

	private JButton showAttributeClosure = null;

	public TableMainPanel() {
		super();

		BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(layout);

		JPanel nfInfoPanel = new JPanel(new GridBagLayout());
		nfInfoPanel.setBorder(BorderFactory.createTitledBorder("Tablename and Normalform"));
		JLabel tblNameCaptionLbl = new JLabel("Table name:");
		tblNameCaptionLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		this.tableNameLbl = new JLabel("");
		JLabel nfCaptionLbl = new JLabel("Normal Form:");
		nfCaptionLbl.setForeground(DBNormalizerColors.COLOR_CAPTION_LABEL);
		this.nfLbl = new JLabel("?");
		this.nfLbl.setOpaque(true);
		this.nfWarningLabel = new JLabel("(No FDs given)");
		this.nfWarningLabel.setOpaque(true);
		this.nfWarningLabel.setBackground(DBNormalizerColors.COLOR_WARN);
		this.nfWarningLabel.setVisible(false);
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 0;
		nfInfoPanel.add(tblNameCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		nfInfoPanel.add(this.tableNameLbl, c);
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		c.weightx = 0;
		nfInfoPanel.add(nfCaptionLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.weightx = 0;
		nfInfoPanel.add(this.nfLbl, c);
		c.gridx = GridBagConstraints.RELATIVE;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1;
		nfInfoPanel.add(this.nfWarningLabel, c);

		this.add(nfInfoPanel);

		this.tableTabbedPanel = new TableTabbedPanel();
		this.tableTabbedPanel.setTableMainPanel(this);

		this.add(this.tableTabbedPanel);

		JPanel keyPanel = new JPanel(new GridBagLayout());
		c = new GridBagConstraints();
		c.insets = new Insets(2, 2, 2, 2);
		keyPanel.setBorder(BorderFactory.createTitledBorder("Keys"));

		this.keyWarningLbl = new JLabel("");
		this.keyWarningLbl.setOpaque(true);
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 2;
		c.gridheight = 1;
		c.weightx = 1.0;
		c.weighty = 0.0;
		c.fill = GridBagConstraints.HORIZONTAL;
		keyPanel.add(this.keyWarningLbl, c);

		// this.candKeyList = new JList(new DefaultListModel());
		// JScrollPane candKeyScrollPane = new JScrollPane(this.candKeyList);
		// keyPanel.add(candKeysCaption);
		// keyPanel.add(candKeyScrollPane);
		this.candKeyList = new JList(new DefaultListModel());
		this.candKeyList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.candKeyList.addMouseListener(new MouseAdapter() {
			@SuppressWarnings("unchecked")
			@Override
			public void mouseClicked(MouseEvent e) {
				// only react on double-clicks
				if (e.getClickCount() == 2) {
					List<String> attrs = (List<String>) candKeyList.getSelectedValue();
					// set a new pk consisting of the selected key's attributes
					PK pk = new PK();
					pk.setAttributes(new ArrayList<String>(attrs));
					table.setPk(pk);
					refreshKeys(false, true);
				}
			}
		});
		JScrollPane candKeyScrollPane = new JScrollPane(this.candKeyList);
		// candKeyScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		// candKeyScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		JPanel candKeyPane = new JPanel(new BorderLayout());
		candKeyPane.setBorder(BorderFactory.createTitledBorder("Candidate Keys"));
		candKeyPane.add(candKeyScrollPane, BorderLayout.CENTER);

		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0.5;
		c.weighty = 1.0;
		c.fill = GridBagConstraints.BOTH;
		keyPanel.add(candKeyPane, c);

		this.pkList = new JList(new DefaultListModel());
		JScrollPane pkScrollPane = new JScrollPane(this.pkList);
		JPanel pkPane = new JPanel(new BorderLayout());
		pkPane.setBorder(BorderFactory.createTitledBorder("PK"));
		pkPane.add(pkScrollPane, BorderLayout.CENTER);

		c.gridx = 1;
		c.gridy = 1;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0.5;
		c.weighty = 1.0;
		c.fill = GridBagConstraints.BOTH;

		keyPanel.add(pkPane, c);

		this.add(keyPanel);

		JPanel fdButtonPanel = new JPanel();

		this.normProposalButton = new JButton("Normalization Proposal");
		this.normProposalButton.setEnabled(false);
		this.normProposalButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				createNormalizationProposal();
			}

		});
		fdButtonPanel.add(this.normProposalButton);

		this.showAttributeClosure = new JButton("Show AttributeClosure");
		this.showAttributeClosure.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				showAttributeClosure(e);
			}

		});
		fdButtonPanel.add(this.showAttributeClosure);

		this.add(fdButtonPanel);
	}

	public void setSession(DBNormalizerSession session) {
		this.session = session;
		this.tableTabbedPanel.setSession(session);
	}

	public void setTable(Table table) {
		this.table = table;

		if (table.getFDMinCover().size() == 0) {
			table.setFDMinCover(FDAlgorithms.calculateMinCover(table.getFDs()));
			FDAlgorithms.setKeysForTable(table, null);
		}

		NFDeterminationResult nfDeterminationResult = table.getNfDeterminationResult();
		if (nfDeterminationResult == null) {
			NFDetermination nfDetermination = new NFDetermination(table);
			try {
				nfDeterminationResult = nfDetermination.determine();
			} catch (DBNormalizerException e) {
				// an error should not occur here, so we just log the error
				logger.error("Unexpected exception while computing normal form", e);
			}
			table.setNfDeterminationResult(nfDeterminationResult);
		}

		refreshKeys(true, true);

		tableNameLbl.setText(table.getName());
		setNF(nfDeterminationResult.getNf());

		tableTabbedPanel.setTable(table);
	}

	public void refreshKeys(boolean refreshCandKeys, boolean refreshPk) {
		if (refreshCandKeys) {
			DefaultListModel listModel = (DefaultListModel) candKeyList.getModel();
			listModel.clear();
			for (CandKey candidateKey : table.getCandKeys()) {
				listModel.addElement(candidateKey.getAttributes());
			}
		}

		PK pk = table.getPk();
		if (refreshPk) {
			DefaultListModel pklistModel = (DefaultListModel) pkList.getModel();
			pklistModel.clear();
			if (pk != null && pk.getAttributes().size() > 0) {
				for (String attr : pk.getAttributes()) {
					pklistModel.addElement(attr);
				}
			}
		}

		PK dbPk = table.getDbPK();
		// look if the pk defined in the db equals the currently set pk
		if (dbPk != null && dbPk.getAttributes().size() > 0) {
			boolean pkOk = false;
			if (dbPk.getAttributes().containsAll(pk.getAttributes())
					&& dbPk.getAttributes().size() == pk.getAttributes().size()) {
				pkOk = true;
			}

			if (!pkOk) {
				this.keyWarningLbl.setText("The currently set PK does not match PK '" + dbPk
						+ "' of the table definition.");
				this.keyWarningLbl.setBackground(DBNormalizerColors.COLOR_ERROR);
			} else {
				this.keyWarningLbl.setText("The PK corresponds to the table definition");
				this.keyWarningLbl.setBackground(DBNormalizerColors.COLOR_OK);
			}
		} else if (table.isDbTable()) { // only show this warning if table is a db-table
			this.keyWarningLbl.setText("The table definition does not contain a PK");
			this.keyWarningLbl.setBackground(DBNormalizerColors.COLOR_WARN);
		} else if (!table.isDbTable()) {
			this.keyWarningLbl.setText("");
			this.keyWarningLbl.setBackground(new Color(Color.OPAQUE));
		}

	}

	private void setNF(NormalForm nf) {
		this.nfLbl.setText(nf.toString());
		Color color = null;
		switch (nf) {
		case BCNF:
			color = DBNormalizerColors.COLOR_OK;
			break;
		case NF3:
			color = DBNormalizerColors.COLOR_WARN;
			break;
		default:
			color = DBNormalizerColors.COLOR_ERROR;
			break;
		}
		this.nfLbl.setBackground(color);

		if (table.getFDs().size() == 0) {
			// show warning, if no FDs are given
			this.nfWarningLabel.setVisible(true);
		} else {
			// hide warning, otherwise
			this.nfWarningLabel.setVisible(false);
		}

		if (nf.compareTo(NormalForm.NF3) < 0) {
			// enable "normalization proposal" button, if NF < 3NF
			this.normProposalButton.setEnabled(true);
		} else {
			this.normProposalButton.setEnabled(false);
		}

	}

	public void fdsChanged() {
		try {
			this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			
			Set<FD> minCover = FDAlgorithms.calculateMinCover(table.getFDs());
			table.setFDMinCover(minCover);
	
			FDAlgorithms.setKeysForTable(table, null);
			NFDetermination nfDetermination = new NFDetermination(table);
			NFDeterminationResult result = null;
			try {
				result = nfDetermination.determine();
			} catch (DBNormalizerException e) {
				// an error should not occur here, so we just log the error
				logger.error("Unexpected exception while computing normal form", e);
			}
			table.setNfDeterminationResult(result);
	
			refreshKeys(true, true);
			setNF(result.getNf());
	
			this.tableTabbedPanel.getFdAnalysisPanel().refresh();
		} catch (Exception e) {
			logger.error("An unexpected exception occured", e);
		} finally {
			this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}

	private void showAttributeClosure(ActionEvent e) {
		InformationPanel infoPanel = this.applicationFrame.getInfoPanel();
		String strAttrs = JOptionPane.showInputDialog("Enter attributes (comma-seperated)");
		AttributeStringSet attrs;
		try {
			attrs = MiscUtil.parseAttributeStringSet(strAttrs, table);
			AttributeStringSet attributeClosure = FDAlgorithms.calculateAttributeClosure(table
					.getFDs(), attrs);
			StringBuffer strAttributeClosure = new StringBuffer();
			strAttributeClosure.append("AttributeClosure(");
			strAttributeClosure.append(attrs.toString());
			strAttributeClosure.append(") = ");
			strAttributeClosure.append(attributeClosure.toString());

			infoPanel.println(strAttributeClosure.toString());
		} catch (DBNormalizerException ex) {
			// do nothing (only the corresponding log message is shown)
		}

	}

	private void createNormalizationProposal() {
		try {
			this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			
			PK pk = table.getPk();
	
			if (pk == null || pk.getAttributes().size() == 0) {
				JOptionPane.showMessageDialog(TableMainPanel.this.applicationFrame,
						"Please select a primary key first!");
				return;
			}
	
			ThreeNFSynthesisAlgorithmKemper normalizer = new ThreeNFSynthesisAlgorithmKemper();
	
			try {
				NormalizationProposal normProposal = normalizer.normalize(table);
				table.setNormProposal(normProposal);
	
				this.applicationFrame.getTableTree().addNormalizationProposal(normProposal);
			} catch (DBNormalizerException e) {
				// an error should not occur here, so we just log the error
				logger.error("Unexpected exception while computing normalization proposal", e);
			}
		} catch (Exception e) {
			logger.error("An unexpected exception occurred", e);
		} finally {
			this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}

	public void setApplicationFrame(ApplicationFrame applicationFrame) {
		this.applicationFrame = applicationFrame;
	}
}
