package de.fesenmeyer.dbnormalizer.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import de.fesenmeyer.dbnormalizer.core.AbstractDBItem;
import de.fesenmeyer.dbnormalizer.core.DBNormalizerSession;
import de.fesenmeyer.dbnormalizer.core.Table;
import de.fesenmeyer.dbnormalizer.core.TableContainer;
import de.fesenmeyer.dbnormalizer.core.TableContainerType;
import de.fesenmeyer.dbnormalizer.core.dbaccess.DatabaseMetadataUtil;
import de.fesenmeyer.dbnormalizer.core.log.DBNormalizerLogger;
import de.fesenmeyer.dbnormalizer.core.util.StringUtil;

/**
 * @author DF
 *
 */
@SuppressWarnings("serial")
public class TableTreePanel extends JPanel {

	private static DBNormalizerLogger logger = DBNormalizerLogger.getLogger(TableTreePanel.class);

	private DBNormalizerSession session = null;

	private DBTableSelectionFrame dbTableSelectionFrame = null;

	private TableTree tableTree = null;

	private JButton addButton = null;

	private JButton removeButton = null;

	private enum Add_Mode {
		CONTAINER, TABLE, ATTRIBUTE, FROM_DB
	};

	private Add_Mode addMode = null;

	public TableTreePanel() {
		super(new BorderLayout());

		JPanel buttonPanel = createButtonPanel();
		this.add(buttonPanel, BorderLayout.NORTH);

		this.tableTree = new TableTree();
		this.tableTree.addTreeSelectionListener(new TreeSelectionListener() {

			public void valueChanged(TreeSelectionEvent e) {
				treeSelectionChanged(e);
			}

		});

		JScrollPane tableTreeScrollPane = new JScrollPane(this.tableTree);
		this.add(tableTreeScrollPane, BorderLayout.CENTER);

	}

	private JPanel createButtonPanel() {
		JPanel buttonPanel = new JPanel();

		this.addButton = new JButton("Add ...");
		this.addButton.setEnabled(false);
		this.addButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addButtonClicked();
			}
		});

		this.removeButton = new JButton("Remove");
		this.removeButton.setEnabled(false);
		this.removeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeButtonClicked();
			}
		});

		buttonPanel.add(addButton);
		buttonPanel.add(removeButton);

		return buttonPanel;
	}

	private Add_Mode getInitialAddMode() {
		Add_Mode initialAddMode = null;
		if (this.session.getDbConfig() == null) {
			initialAddMode = Add_Mode.CONTAINER;
		} else {
			initialAddMode = Add_Mode.FROM_DB;
		}
		return initialAddMode;
	}

	private void treeSelectionChanged(TreeSelectionEvent e) {
		if (this.session != null) {
			if (this.session.getDbConfig() == null) {
				TreePath path = e.getNewLeadSelectionPath();
				if (path == null) {
					this.addMode = Add_Mode.CONTAINER;
				} else {
					DBNormalizerTreeNode node = (DBNormalizerTreeNode) path.getLastPathComponent();
					DBNormalizerTreeNode.NodeType nodeType = node.getNodeType();

					switch (nodeType) {
					case TABLE:
					case ATTRIBUTE:
						this.addMode = Add_Mode.ATTRIBUTE;
						break;
					case TABLE_CONTAINER:
						this.addMode = Add_Mode.TABLE;
						break;
					case ROOT_NODE:
						this.addMode = Add_Mode.CONTAINER;
						break;
					}
				}
				this.addButton.setText(getAddTitle(this.addMode));
			}
		}
	}

	private void addButtonClicked() {
		if (addMode == Add_Mode.CONTAINER) {
			// TODO currently it is not possible to add containers other than schema
			String strContainerNames = JOptionPane
					.showInputDialog("Enter schema names (comma-seperated)");
			if (strContainerNames != null) {
				String[] containerNames = StringUtil.parseStringArray(strContainerNames);

				for (String containerName : containerNames) {
					TableContainer tableContainer = new TableContainer(containerName,
							TableContainerType.SCHEMA);
					tableTree.addContainer(tableContainer);
				}
			}
		} else if (addMode == Add_Mode.TABLE) {
			String strTableNames = JOptionPane
					.showInputDialog("Enter table names (comma-seperated)");
			if (strTableNames != null) {
				String[] tableNames = StringUtil.parseStringArray(strTableNames);
				for (String tableName : tableNames) {
					TreePath path = tableTree.getSelectionPath();
					tableTree.addTable((DBNormalizerTreeNode) path.getLastPathComponent(),
							tableName);
				}
			}
		} else if (addMode == Add_Mode.ATTRIBUTE) {
			String strAttrNames = JOptionPane
					.showInputDialog("Enter attribute names (comma-seperated)");
			if (strAttrNames != null) {
				String[] attrNames = StringUtil.parseStringArray(strAttrNames);
				TreePath path = tableTree.getSelectionPath();
				tableTree.addAttributes((DBNormalizerTreeNode) path.getLastPathComponent(),
						attrNames);
			}
		} else if (addMode == Add_Mode.FROM_DB) {
			if (this.dbTableSelectionFrame == null) {
				this.dbTableSelectionFrame = new DBTableSelectionFrame((JFrame) this
						.getTopLevelAncestor());
				this.dbTableSelectionFrame.setSession(session);
			}
			this.dbTableSelectionFrame.setVisible(true);
			boolean accepted = this.dbTableSelectionFrame.isAccepted();
			if (accepted) {
				List<AbstractDBItem> dbItems = this.dbTableSelectionFrame.getSelectedItems();
				for (AbstractDBItem dbItem : dbItems) {
					if (dbItem instanceof TableContainer) {
						TableContainer tableContainer = (TableContainer) dbItem;
						Collection<Table> tablesFromDB;
						try {
							tablesFromDB = DatabaseMetadataUtil.getTableInfos(session,
									tableContainer, true);
							for (Table tableFromDB : tablesFromDB) {
								this.session.addTable2Normalize(tableContainer, tableFromDB);
							}
						} catch (SQLException e) {
							logger.error("Unexpected SQL Exception", e);
						}

					} else if (dbItem instanceof Table) {
						Table table = (Table) dbItem;
						this.session.addTable2Normalize(table.getContainer(), table);
					}
				}
				// if there has sth been added
				if (dbItems.size() > 0) {
					DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) this.tableTree
							.getModel().getRoot();

					Enumeration<TreePath> expandedPaths = this.tableTree
							.getExpandedDescendants(new TreePath(rootNode.getPath()));
					LinkedList<Object[]> expandedObjectsList = new LinkedList<Object[]>();
					if (expandedPaths != null) {
						while (expandedPaths.hasMoreElements()) {
							TreePath expandedPath = expandedPaths.nextElement();
							Object[] objects = ((DefaultMutableTreeNode) expandedPath
									.getLastPathComponent()).getUserObjectPath();
							expandedObjectsList.add(objects);
						}
					}

					TreePath[] selectedPaths = this.tableTree.getSelectionPaths();
					LinkedList<Object[]> selectedObjectsList = new LinkedList<Object[]>();
					if (selectedPaths != null) {
						for (int i = 0; i < selectedPaths.length; i++) {
							TreePath selectedPath = selectedPaths[i];
							Object[] objects = ((DefaultMutableTreeNode) selectedPath
									.getLastPathComponent()).getUserObjectPath();
							selectedObjectsList.add(objects);
						}
					}

					this.tableTree.setSession(session);

					rootNode = (DefaultMutableTreeNode) this.tableTree.getModel().getRoot();
					recreateExpansionOrSelection(rootNode, 0, expandedObjectsList, true);
					recreateExpansionOrSelection(rootNode, 0, selectedObjectsList, false);

				}

			}
		}
	}

	private void recreateExpansionOrSelection(DefaultMutableTreeNode node, int level,
			List<Object[]> objectsList, boolean expand) {
		LinkedList<Object[]> newList = new LinkedList<Object[]>();
		for (Object[] objects : objectsList) {
			Object o = objects[level];
			if (node.getUserObject().equals(o)) {
				if (level == objects.length - 1) {
					TreePath path = new TreePath(node.getPath());
					if (expand) {
						logger
								.debug("Found previously expanded node with path: "
										+ path.toString());
						this.tableTree.expandPath(path);
					} else {
						logger
								.debug("Found previously selected node with path: "
										+ path.toString());
						this.tableTree.addSelectionPath(path);
					}

				} else {
					newList.add(objects);
				}
			}
		}
		if (node.getChildCount() > 0) {
			for (int i = 0; i < node.getChildCount(); i++) {
				DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) node.getChildAt(i);
				recreateExpansionOrSelection(childNode, level + 1, newList, expand);
			}
		}
	}

	private void removeButtonClicked() {
		TreePath selPath = tableTree.getSelectionPath();
		tableTree.remove(selPath);
	}

	private String getAddTitle(Add_Mode addMode) {
		String str = "";
		if (addMode == Add_Mode.TABLE) {
			str = "Add Table";
		} else if (addMode == Add_Mode.ATTRIBUTE) {
			str = "Add Attribute";
		} else if (addMode == Add_Mode.CONTAINER) {
			str = "Add Container";
		} else if (addMode == Add_Mode.FROM_DB) {
			str = "Add ...";
		} else {
			// is needed to set an initial value
			// if there is no session
			str = "Add ...";
		}
		return str;
	}

	public TableTree getTableTree() {
		return tableTree;
	}

	public void changeSession(DBNormalizerSession session) {
		this.session = session;

		Add_Mode initialAddMode = getInitialAddMode();
		this.addMode = initialAddMode;
		this.addButton.setText(getAddTitle(initialAddMode));

		// enable the add and remove buttons
		this.addButton.setEnabled(true);
		this.removeButton.setEnabled(true);

		this.tableTree.setSession(session);

		this.dbTableSelectionFrame = null;
	}

}
